# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""This module contains these classes: ``RMXprtModule`` and ``Rmxprt``."""

from __future__ import absolute_import  # noreorder

from pyaedt.application.AnalysisRMxprt import FieldAnalysisRMxprt
from pyaedt.generic.general_methods import pyaedt_function_handler
from pyaedt.modeler.cad.elements3d import BinaryTreeNode
from pyaedt.modules.SetupTemplates import SetupKeys


class RMXprtModule(object):
    """Provides RMxprt module properties."""

    component = None

    def __init__(self, app):
        self._app = app
        self.oeditor = app.oeditor

    @property
    def properties(self):
        """Object history.

        Returns
        -------
            :class:`pyaedt.modeler.cad.elements3d.BinaryTree` when successful,
            ``False`` when failed.

        """
        try:
            child_object = self._app.odesign.GetChildObject("Machine")
            if self.component:
                child_object = child_object.GetChildObject(self.component)
            parent = BinaryTreeNode(self.component if self.component else "Machine", child_object, False)
            return parent
        except Exception:
            return False

    @pyaedt_function_handler()
    def __setitem__(self, parameter_name, value):
        def _apply_val(dict_in, name, value):
            if name in dict_in.props:
                if (
                    isinstance(dict_in.props[name], list)
                    and ":=" in dict_in.props[name][0]
                    and not isinstance(value, list)
                ):
                    prps = dict_in.props[name][::]
                    prps[1] = value
                    value = prps
                dict_in.props[name] = value
                return True
            else:
                for _, child in dict_in.children.items():
                    ret = _apply_val(child, name, value)
                    if ret:
                        return True

        if self.properties:
            _apply_val(self.properties, parameter_name, value)
        else:  # pragma: no cover
            self._app.logger.error("Properties not available for {}".format(self.component))

    @pyaedt_function_handler()
    def __getitem__(self, parameter_name):
        def _get_val(dict_in, name):
            if name in dict_in.props:
                return dict_in.props[name]
            else:
                for _, child in dict_in.children.items():
                    return _get_val(child, name)

        if self.properties:
            return _get_val(self.properties, parameter_name)
        else:  # pragma: no cover
            self._app.logger.error("Properties not available for {}".format(self.component))


class Stator(RMXprtModule):
    """Provides stator properties."""

    component = "Stator"


class Rotor(RMXprtModule):
    """Provides rotor properties."""

    component = "Rotor"


class Shaft(RMXprtModule):
    """Provides rotor properties."""

    component = "Shaft"


class Machine(RMXprtModule):
    """Provides rotor properties."""

    component = ""


class Circuit(RMXprtModule):
    """Provides rotor properties."""

    component = "Circuit"


class Rmxprt(FieldAnalysisRMxprt):
    """Provides the RMxprt app interface.

    Parameters
    ----------
    project : str, optional
        Name of the project to select or the full path to the project
        or AEDTZ archive to open. The default is ``None``, in which
        case an attempt is made to get an active project. If no
        projects are present, an empty project is created.
    design : str, optional
        Name of the design to select. The default is ``None``, in
        which case an attempt is made to get an active design. If no
        designs are present, an empty design is created.
    solution_type : str, optional
        Solution type to apply to the design. The default is
        ``None``, in which case the default type is applied.
    model_units : str, optional
        Model units.
    setup : str, optional
        Name of the setup to use as the nominal. The default is
        ``None``, in which case the active setup is used or
        nothing is used.
    version : str, int, float, optional
        Version of AEDT to use. The default is ``None``, in which case
        the active setup is used or the latest installed version is
        used.
        Examples of input values are ``232``, ``23.2``,``2023.2``,``"2023.2"``.
    non_graphical : bool, optional
        Whether to launch AEDT in non-graphical mode. The default
        is ``False``, in which case AEDT is launched in graphical mode.
        This parameter is ignored when a script is launched within AEDT.
    new_desktop : bool, optional
        Whether to launch an instance of AEDT in a new thread, even if
        another instance of the ``specified_version`` is active on the
        machine.  The default is ``False``.
    close_on_exit : bool, optional
        Whether to release AEDT on exit. The default is ``False``.
    student_version : bool, optional
        Whether to open the AEDT student version. The default is ``False``.
    machine : str, optional
        Machine name to connect the oDesktop session to. This works only in 2022 R2 or
        later. The remote server must be up and running with the command
        `"ansysedt.exe -grpcsrv portnum"`. If the machine is `"localhost"`, the
        server also starts if not present.
    port : int, optional
        Port number on which to start the oDesktop communication on an already existing server.
        This parameter is ignored when creating a new server. It works only in 2022 R2 or later.
        The remote server must be up and running with the command `"ansysedt.exe -grpcsrv portnum"`.
    aedt_process_id : int, optional
        Process ID for the instance of AEDT to point PyAEDT at. The default is
        ``None``. This parameter is only used when ``new_desktop = False``.
    remove_lock : bool, optional
        Whether to remove lock to project before opening it or not.
        The default is ``False``, which means to not unlock
        the existing project if needed and raise an exception.

    Examples
    --------
    Create an instance of RMxprt and connect to an existing RMxprt
    design or create a new RMxprt design if one does not exist.

    >>> from pyaedt import Rmxprt
    >>> app = Rmxprt()

    Create an instance of Rmxprt and link to a project named
    ``"projectname"``. If this project does not exist, create one with
    this name.

    >>> app = Rmxprt(projectname)

    Create an instance of RMxprt and link to a design named
    ``"designname"`` in a project named ``"projectname"``.

    >>> app = Rmxprt(projectname,designame)

    Create an instance of RMxprt and open the specified project,
    which is ``"myfile.aedt"``.

    >>> app = Rmxprt("myfile.aedt")
    """

    @pyaedt_function_handler(
        designname="design",
        projectname="project",
        specified_version="version",
        setup_name="setup",
        new_desktop_session="new_desktop",
    )
    def __init__(
        self,
        project=None,
        design=None,
        solution_type=None,
        model_units=None,
        setup=None,
        version=None,
        non_graphical=False,
        new_desktop=False,
        close_on_exit=False,
        student_version=False,
        machine="",
        port=0,
        aedt_process_id=None,
        remove_lock=False,
    ):
        FieldAnalysisRMxprt.__init__(
            self,
            "RMxprtSolution",
            project,
            design,
            solution_type,
            setup,
            version,
            non_graphical,
            new_desktop,
            close_on_exit,
            student_version,
            machine,
            port,
            aedt_process_id,
            remove_lock=remove_lock,
        )
        if not model_units or model_units == "mm":
            model_units = "mm"
        elif model_units != "in":
            raise AssertionError("Invalid model units string {}.".format(model_units))
        self.modeler.oeditor.SetMachineUnits(["NAME:Units Parameter", "Units:=", model_units, "Rescale:=", False])
        self.general = Machine(self)
        self.stator = Stator(self)
        self.rotor = Rotor(self)
        self.shaft = Shaft(self)
        self.circuit = Circuit(self)

    def _init_from_design(self, *args, **kwargs):
        self.__init__(*args, **kwargs)

    @property
    def design_type(self):
        """Machine design type."""
        return self.design_solutions.design_type

    @design_type.setter
    def design_type(self, value):
        self.design_solutions.design_type = value

    @pyaedt_function_handler(name="name", setuptype="setup_type")
    def create_setup(self, name="MySetupAuto", setup_type=None, **kwargs):
        """Create an analysis setup for RmXport.

        Optional arguments are passed along with the ``setup_type`` and ``name``
        parameters. Keyword names correspond to the ``setup_type``
        corresponding to the native AEDT API.  The list of
        keywords here is not exhaustive.

        Parameters
        ----------
        name : str, optional
            Name of the setup. The default is "Setup1".
        setup_type : int, str, optional
            Type of the setup. Options are "IcepakSteadyState"
            and "IcepakTransient". The default is "IcepakSteadyState".
        **kwargs : dict, optional
            Available keys depend on the setup chosen.
            For more information, see :doc:`../SetupTemplatesRmxprt`.

        Returns
        -------
        :class:`pyaedt.modules.SolveSetup.SetupHFSS`
            Solver Setup object.

        References
        ----------

        >>> oModule.InsertSetup

        Examples
        --------

        >>> from pyaedt import Hfss
        >>> hfss = Hfss()
        >>> hfss.create_setup(name="Setup1",setup_type="HFSSDriven",Frequency="10GHz")

        """
        if setup_type is None:
            setup_type = self.design_solutions.default_setup
        elif setup_type in SetupKeys.SetupNames:
            setup_type = SetupKeys.SetupNames.index(setup_type)
        if "props" in kwargs:
            return self._create_setup(name=name, setup_type=setup_type, props=kwargs["props"])
        else:
            setup = self._create_setup(name=name, setup_type=setup_type)
        setup.auto_update = False
        for arg_name, arg_value in kwargs.items():
            if setup[arg_name] is not None:
                setup[arg_name] = arg_value
        setup.auto_update = True
        setup.update()
        return setup

    @pyaedt_function_handler()
    def export_configuration(self, output_file):
        """Export Rmxprt project to config file.

        Parameters
        ----------
        output_file : str
            Full path to json file to be created.

        Returns
        -------
        str
           Full path to json file created.
        """

        def jsonalize(dict_in, dict_out):
            dict_out[dict_in.node] = {}
            for k, v in dict_in.props.items():
                if not k.endswith("/Choices"):
                    dict_out[dict_in.node][k] = v
            for _, c in dict_in.children.items():
                jsonalize(c, dict_out)

        new_dict = {}
        jsonalize(self.general.properties, new_dict)
        jsonalize(self.stator.properties, new_dict)
        jsonalize(self.rotor.properties, new_dict)
        jsonalize(self.circuit.properties, new_dict)
        jsonalize(self.shaft.properties, new_dict)
        from pyaedt.generic.general_methods import write_configuration_file

        write_configuration_file(new_dict, output_file)
        return output_file

    @pyaedt_function_handler()
    def import_configuration(self, input_file):
        """Parse a json file and assign all the properties to the Rmxprt design.

        Parameters
        ----------
        input_file : str
            Full path to json file to be used.

        Returns
        -------
        bool
            ``True`` when successful, ``False`` when failed.
        """

        from pyaedt.generic.general_methods import read_configuration_file

        new_dict = read_configuration_file(input_file)
        for k, v in new_dict.items():
            dictin = self.stator
            if k == "General":
                dictin = self.general
            elif k == "Circuit":
                dictin = self.circuit
            elif k in ["Rotor", "Pole"]:
                dictin = self.rotor
            elif k == "Shaft":
                dictin = self.shaft
            for i, j in v.items():
                dictin[i] = j
        return True



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import POINTER
from ctypes import byref
from ctypes import c_bool
from ctypes import c_char_p
from ctypes import c_double
from ctypes import c_int
from enum import Enum
import math

import pyaedt
from pyaedt.filtersolutions_core.graph_setup import GraphSetup


class FrequencyResponseColumn(Enum):
    """Provides an enum of frequency response parameters.

    **Attributes:**

    - MAGNITUDE_DB: Represents the frequency response magnitude in dB.
    - PHASE_DEG: Represents the frequency response phase in degree.
    - GROUP_DELAY: Represents the frequency response group delay.
    - PHASE_RAD: Represents the frequency response phase in radian.
    - MAGNITUDE_ARITH: Represents the frequency response magnitude.
    - MAGNITUDE_REAL: Represents the real part of frequency response magnitude.
    - MAGNITUDE_IMAG: Represents the imaginary part of frequency response magnitude.
    - PHASE_DEV_DEG: Represents the frequency response phase deviation in degrees.
    - PHASE_DEV_RAD: Represents the frequency response phase deviation in radian.
    - FREQUENCY: Represents frequency parameter of the frequency response .
    """

    MAGNITUDE_DB = 0
    PHASE_DEG = 1
    GROUP_DELAY = 2
    PHASE_RAD = 4
    MAGNITUDE_ARITH = 3
    MAGNITUDE_REAL = 5
    MAGNITUDE_IMAG = 6
    PHASE_DEV_DEG = 7
    PHASE_DEV_RAD = 8
    FREQUENCY = 9


class TimeResponseColumn(Enum):
    """Provides an enum of time response parameters.

    **Attributes:**

    - STEP_RESPONSE: Represents the step time response.
    - RAMP_RESPONSE: Represents the ramp time response.
    - IMPULSE_RESPONSE: Represents the impulse time response.
    - STEP_RESPONSE_DB: Represents the step time response in dB.
    - RAMP_RESPONSE_DB: Represents the ramp time response in dB.
    - IMPULSE_RESPONSE_DB: Represents the impulse time response in dB.
    - TIME: Represents time parameter of the time response .
    """

    STEP_RESPONSE = 0
    RAMP_RESPONSE = 1
    IMPULSE_RESPONSE = 2
    STEP_RESPONSE_DB = 3
    RAMP_RESPONSE_DB = 4
    IMPULSE_RESPONSE_DB = 5
    TIME = 6


class SParametersResponseColumn(Enum):
    """Provides an enum of S parameters.

    **Attributes:**

    - S21_DB: Represents the S21 parameter in dB.
    - S11_DB: Represents the S11 parameter in dB.
    - S21_ARITH: Represents the S21 parameter.
    - S11_ARITH: Represents the S11 parameter.
    - FREQUENCY: Represents the S parameters' frequency parameter.
    """

    S21_DB = 0
    S11_DB = 1
    S21_ARITH = 3
    S11_ARITH = 4
    FREQUENCY = 5


class PoleZerosResponseColumn(Enum):
    """Provides an enum of pole zero x and y coordinates of transmission (TX) or reflection (RX) zeros.

    **Attributes:**

    - TX_ZERO_DEN_X: Represents the x coordinate of the filter transmission zero denominator.
    - TX_ZERO_DEN_Y: Represents the y coordinate of the filter transmission zero denominator.
    - PROTO_TX_ZERO_DEN_X: Represents the x coordinate of the prototype filter transmission zero denominator.
    - PROTO_TX_ZERO_DEN_Y: Represents the y coordinate of the prototype filter transmission zero denominator.
    - TX_ZERO_NUM_X: Represents the x coordinate of the filter transmission zero numerator.
    - TX_ZERO_NUM_Y: Represents the y coordinate of the filter transmission zero numerator.
    - PROTO_TX_ZERO_NUM_X: Represents the x coordinate of the prototype filter transmission zero numerator.
    - PROTO_TX_ZERO_NUM_Y: Represents the y coordinate of the prototype filter transmission zero numerator.
    - RX_ZERO_DEN_X: Represents the x coordinate of the filter reflection zero denominator.
    - RX_ZERO_DEN_Y: Represents the y coordinate of the filter reflection zero denominator.
    - PROTO_RX_ZERO_DEN_X: Represents the x coordinate of the prototype filter reflection zero denominator.
    - PROTO_RX_ZERO_DEN_Y: Represents the y coordinate of the prototype filter reflection zero denominator.
    - RX_ZERO_NUM_X: Represents the x coordinate of the filter reflection zero numerator.
    - RX_ZERO_NUM_Y: Represents the y coordinate of the filter reflection zero numerator.
    - PROTO_RX_ZERO_NUM_X: Represents the x coordinate of the prototype filter reflection zero numerator.
    - PROTO_RX_ZERO_NUM_Y: Represents the y coordinate of the prototype filter reflection zero numerator.
    """

    TX_ZERO_DEN_X = 0
    TX_ZERO_DEN_Y = 1
    PROTO_TX_ZERO_DEN_X = 2
    PROTO_TX_ZERO_DEN_Y = 3
    TX_ZERO_NUM_X = 4
    TX_ZERO_NUM_Y = 5
    PROTO_TX_ZERO_NUM_X = 6
    PROTO_TX_ZERO_NUM_Y = 7
    RX_ZERO_DEN_X = 8
    RX_ZERO_DEN_Y = 9
    PROTO_RX_ZERO_DEN_X = 10
    PROTO_RX_ZERO_DEN_Y = 11
    RX_ZERO_NUM_X = 12
    RX_ZERO_NUM_Y = 13
    PROTO_RX_ZERO_NUM_X = 14
    PROTO_RX_ZERO_NUM_Y = 15


class IdealResponse:
    """Returns the data for available ideal filter responses.

    Types of responses Include ``frequency``, ``time``, ``S parameters``, ``transfer function``,
    and ``pole zero location``.

    This class allows you to construct all the necessary ideal response attributes for the ``FilterDesign`` class.
    """

    def __init__(self):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._define_response_dll_functions()
        self.graph_setup = GraphSetup()

    def _define_response_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.getIdealFrequencyResponseSize.argtype = POINTER(c_int)
        self._dll.getIdealFrequencyResponseSize.restype = c_int
        self._dll.getIdealFrequencyResponse.argtypes = [POINTER(c_double), c_int, c_int]
        self._dll.getIdealFrequencyResponse.restype = c_int

        self._dll.getIdealTimeResponseSize.argtype = POINTER(c_int)
        self._dll.getIdealTimeResponseSize.restype = c_int
        self._dll.getIdealTimeResponse.argtypes = [POINTER(c_double), c_int, c_int]
        self._dll.getIdealTimeResponse.restype = c_int

        self._dll.getIdealSParamatersResponseSize.argtype = POINTER(c_int)
        self._dll.getIdealSParamatersResponseSize.restype = c_int
        self._dll.getIdealSParamatersResponse.argtypes = [
            POINTER(c_double),
            c_int,
            c_int,
        ]
        self._dll.getIdealSParamatersResponse.restype = c_int

        self._dll.getIdealPoleZerosResponseSize.argtypes = [POINTER(c_int), c_int]
        self._dll.getIdealPoleZerosResponseSize.restype = c_int
        self._dll.getIdealPoleZerosResponse.argtypes = [POINTER(c_double), c_int, c_int]
        self._dll.getIdealPoleZerosResponse.restype = c_int

        self._dll.getIdealTransferFunctionResponseSize.argtype = POINTER(c_int)
        self._dll.getIdealTransferFunctionResponseSize.restype = c_int
        self._dll.getIdealTransferFunctionResponse.argtypes = [c_char_p, c_int]
        self._dll.getIdealTransferFunctionResponse.restype = c_int

        self._dll.setVSGAnalsyis.argtype = c_bool
        self._dll.setVSGAnalsyis.restype = c_int
        self._dll.getVSGAnalsyis.argtype = POINTER(c_bool)
        self._dll.getVSGAnalsyis.restype = c_int

    def _frequency_response_getter(self, column: FrequencyResponseColumn):
        """Get the ideal filter frequency response.

        Parameters
        ----------
        column: `FrequencyResponseColumn`
            Frequency response column to get.

        Returns
        -------
        list
            List of values for the requested frequency response column (such as magnitude or phase).
        """
        size = c_int()
        status = self._dll.getIdealFrequencyResponseSize(byref(size))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        array = (c_double * size.value)()
        status = self._dll.getIdealFrequencyResponse(array, size.value, column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        values = [float(val) for val in array]
        return values

    def _time_response_getter(self, column: TimeResponseColumn):
        """Get the ideal filter time response.

        Parameters
        ----------
        column: `TimeResponseColumn`
            Time response column to get.

        Returns
        -------
        list
            List of values for the requested time response column (such as step or pulse).
        """
        size = c_int()
        status = self._dll.getIdealTimeResponseSize(byref(size))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        array = (c_double * size.value)()
        status = self._dll.getIdealTimeResponse(array, size.value, column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        values = [float(val) for val in array]
        return values

    def _sparamaters_response_getter(self, column: SParametersResponseColumn):
        """Get the ideal filter S parameter's response.

        Parameters
        ----------
        column: `SParametersResponseColumn`
            S parameter's response column to get.

        Returns
        -------
        list
            List of values for the requested S parameter's response column (such as S11 or S21).
        """
        size = c_int()
        status = self._dll.getIdealSParamatersResponseSize(byref(size))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        array = (c_double * size.value)()
        status = self._dll.getIdealSParamatersResponse(array, size.value, column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        values = [float(val) for val in array]
        return values

    def _pole_zeros_response_getter(self, column: PoleZerosResponseColumn):
        """Get the ideal pole zero's location parameters (such as the x coordinate of
        the denominator of transmission zeros).

        Parameters
        ----------
        column: `PoleZerosResponseColumn`
            Pole zero's response column to get.

        Returns
        -------
        list
            List of values for the requested pole zero's response column.
        """
        size = c_int()
        status = self._dll.getIdealPoleZerosResponseSize(byref(size), column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        array = (c_double * size.value)()
        status = self._dll.getIdealPoleZerosResponse(array, size.value, column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        values = [float(val) for val in array]
        return values

    def transfer_function_response(self):
        """Get the ideal filter transfer function's parameters.

        Returns
        -------
        str
            Requested parameter array.

            str
                Multi-line string where each line contains a coefficient from
                the numerator and/or the denominator of the transfer function.
                The coefficient for the highest-order term is first, and the terms are in decreasing order.
        """
        size = c_int()
        status = self._dll.getIdealTransferFunctionResponseSize(byref(size))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        transfer_function_response_string = self._dll_interface.get_string(
            self._dll.getIdealTransferFunctionResponse, max_size=size.value
        )
        return transfer_function_response_string

    @property
    def vsg_analysis_enabled(self) -> bool:
        """Flag indicating if the offset due to source resistor in frequency and time responses is enabled.

        Returns
        -------
        bool
        """

        vsg_analysis_enabled = c_bool()
        status = self._dll.getVSGAnalsyis(byref(vsg_analysis_enabled))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(vsg_analysis_enabled.value)

    @vsg_analysis_enabled.setter
    def vsg_analysis_enabled(self, filter_vsg_analysis_enabled: bool):
        status = self._dll.setVSGAnalsyis(filter_vsg_analysis_enabled)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def frequency_response(
        self,
        y_axis_parameter=FrequencyResponseColumn.MAGNITUDE_DB,
        minimum_frequency="200 MHz",
        maximum_frequency="5 GHz",
        vsg_analysis_enabled=False,
    ):
        """Get the ideal filter frequency response for the given parameters.

        Parameters
        ----------
        y_axis_parameter: `FrequencyResponseColumn`, optional
             Frequency response column to return. The default is the frequency response magnitude in dB.
        minimum_frequency: str, optional
            The default is ``200 MHz``.
        maximum_frequency: str, optional
            The default is ``5 GHz``.
        vsg_analysis_enabled: bool, optional
            The default is ``False``.

        Returns
        -------
        tuple
            The tuple contains two lists of strings. The first is a list
            of the defined frequency ranges, and the second is a
            list of the requested parameters.
        """
        self.graph_setup.minimum_frequency = minimum_frequency
        self.graph_setup.maximum_frequency = maximum_frequency
        self.vsg_analysis_enabled = vsg_analysis_enabled
        frequency = self._frequency_response_getter(FrequencyResponseColumn.FREQUENCY)
        frequency_hz = []
        for freq in frequency:
            frequency_hz.append(freq / (2 * math.pi))
        parameter = self._frequency_response_getter(y_axis_parameter)
        return frequency_hz, parameter

    def time_response(
        self,
        y_axis_parameter=TimeResponseColumn.STEP_RESPONSE,
        minimum_time="0 s",
        maximum_time="10 ns",
        vsg_analysis_enabled=False,
    ):
        """Get the ideal filter time response for the given parameters.

        Parameters
        ----------
        y_axis_parameter: `TimeResponseColumn`, optional
            Time response column to get. The default is the step time response.
        minimum_time: str, optional
            The default is ``0 s``.
        maximum_time: str, optional
            The default is ``10 ns``.
        vsg_analysis_enabled: bool, optional
            The default is ``False``.

        Returns
        -------
        tuple
            The tuple contains two lists of strings. The first is a list
            of the defined time ranges, and the second is a
            list of the requested parameters.
        """
        self.graph_setup.minimum_time = minimum_time
        self.graph_setup.maximum_time = maximum_time
        self.vsg_analysis_enabled = vsg_analysis_enabled
        time = self._time_response_getter(TimeResponseColumn.TIME)
        parameter = self._time_response_getter(y_axis_parameter)
        return time, parameter

    def s_parameters(
        self,
        y_axis_parameter=SParametersResponseColumn.S21_DB,
        minimum_frequency="200 MHz",
        maximum_frequency="5 GHz",
    ):
        """Get the ideal filter S parameters response for the given parameters.

        Parameters
        ----------
        y_axis_parameter: `SParametersResponseColumn`, optional
            S parameter's response column to get. The default is the S21 parameter response in dB.
        minimum_frequency: str, optional
            The default is ``200 MHz``.
        maximum_frequency: str, optional
            The default is ``5 GHz``.
        vsg_analysis_enabled: bool, optional
            The default is ``False``.

        Returns
        -------
        tuple
            The tuple contains two lists of strings. The first is a list
            of the defined frequency ranges, and the second is a
            list of the requested parameters.
        """
        self.graph_setup.minimum_frequency = minimum_frequency
        self.graph_setup.maximum_frequency = maximum_frequency
        frequency = self._sparamaters_response_getter(SParametersResponseColumn.FREQUENCY)
        frequency_hz = []
        for freq in frequency:
            frequency_hz.append(freq / (2 * math.pi))
        parameter = self._sparamaters_response_getter(y_axis_parameter)
        return frequency_hz, parameter

    def pole_zero_locations(
        self,
        x_axis_parameter=PoleZerosResponseColumn.TX_ZERO_DEN_X,
        y_axis_parameter=PoleZerosResponseColumn.TX_ZERO_DEN_Y,
    ):
        """Get the ideal pole zero location for the given parameters.

        Parameters
        ----------
        x_axis_parameter: `PoleZerosResponseColumn`, optional
            X axis parameter of the pole zeros response column to get. The default is the x coordinate
            of the filter transmission zero denominator.
        y_axis_parameter: `PoleZerosResponseColumn`, optional
            Y axis parameter of the pole zeros response column to get. The default is the y coordinate
            of the filter transmission zero denominator.

        Returns
        -------
        tuple
            The tuple contains two lists of strings. The first is a list
            of the x coordinates of the requested parameter, and the second is a
            list of the y coordinates of the requested parameter.
        """
        x_parameter = self._pole_zeros_response_getter(x_axis_parameter)
        y_parameter = self._pole_zeros_response_getter(y_axis_parameter)
        return x_parameter, y_parameter



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import c_char_p
from ctypes import c_int

import pyaedt


class LumpedParasitics:
    """Defines attributes of the lumped element parasitics.

    This class lets you to construct all the necessary parasitic
    attributes of lumped elements for the ``LumpedDesign`` class.
    """

    def __init__(self):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._define_parasitics_dll_functions()

    def _define_parasitics_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.setLumpedCapacitorQ.argtype = c_char_p
        self._dll.setLumpedCapacitorQ.restype = c_int
        self._dll.getLumpedCapacitorQ.argtypes = [c_char_p, c_int]
        self._dll.getLumpedCapacitorQ.restype = c_int

        self._dll.setLumpedCapacitorRs.argtype = c_char_p
        self._dll.setLumpedCapacitorRs.restype = c_int
        self._dll.getLumpedCapacitorRs.argtypes = [c_char_p, c_int]
        self._dll.getLumpedCapacitorRs.restype = c_int

        self._dll.setLumpedCapacitorRp.argtype = c_char_p
        self._dll.setLumpedCapacitorRp.restype = c_int
        self._dll.getLumpedCapacitorRp.argtypes = [c_char_p, c_int]
        self._dll.getLumpedCapacitorRp.restype = c_int

        self._dll.setLumpedCapacitorLs.argtype = c_char_p
        self._dll.setLumpedCapacitorLs.restype = c_int
        self._dll.getLumpedCapacitorLs.argtypes = [c_char_p, c_int]
        self._dll.getLumpedCapacitorLs.restype = c_int

        self._dll.setLumpedInductorQ.argtype = c_char_p
        self._dll.setLumpedInductorQ.restype = c_int
        self._dll.getLumpedInductorQ.argtypes = [c_char_p, c_int]
        self._dll.getLumpedInductorQ.restype = c_int

        self._dll.setLumpedInductorRs.argtype = c_char_p
        self._dll.setLumpedInductorRs.restype = c_int
        self._dll.getLumpedInductorRs.argtypes = [c_char_p, c_int]
        self._dll.getLumpedInductorRs.restype = c_int

        self._dll.setLumpedInductorRp.argtype = c_char_p
        self._dll.setLumpedInductorRp.restype = c_int
        self._dll.getLumpedInductorRp.argtypes = [c_char_p, c_int]
        self._dll.getLumpedInductorRp.restype = c_int

        self._dll.setLumpedInductorCp.argtype = c_char_p
        self._dll.setLumpedInductorCp.restype = c_int
        self._dll.getLumpedInductorCp.argtypes = [c_char_p, c_int]
        self._dll.getLumpedInductorCp.restype = c_int

    @property
    def capacitor_q(self) -> str:
        """Q factor value of non-ideal capacitors in the synthesized circuit.
        The default is ``infinite``.

        Returns
        -------
        str
        """
        capacitor_q_string = self._dll_interface.get_string(self._dll.getLumpedCapacitorQ)
        return capacitor_q_string

    @capacitor_q.setter
    def capacitor_q(self, capacitor_q_string):
        self._dll_interface.set_string(self._dll.setLumpedCapacitorQ, capacitor_q_string)

    @property
    def capacitor_rs(self) -> str:
        """Series resistor value of non-ideal capacitors in the synthesized circuit.
        The default is ``0``.

        Returns
        -------
        str
        """
        capacitor_rs_string = self._dll_interface.get_string(self._dll.getLumpedCapacitorRs)
        return capacitor_rs_string

    @capacitor_rs.setter
    def capacitor_rs(self, capacitor_rs_string):
        self._dll_interface.set_string(self._dll.setLumpedCapacitorRs, capacitor_rs_string)

    @property
    def capacitor_rp(self) -> str:
        """Shunt resistor value of non-ideal capacitors in the synthesized circuit.
        The default is ``infinite``.

        Returns
        -------
        str
        """
        capacitor_rp_string = self._dll_interface.get_string(self._dll.getLumpedCapacitorRp)
        return capacitor_rp_string

    @capacitor_rp.setter
    def capacitor_rp(self, capacitor_rp_string):
        self._dll_interface.set_string(self._dll.setLumpedCapacitorRp, capacitor_rp_string)

    @property
    def capacitor_ls(self) -> str:
        """Series inductance value of non-ideal capacitors in the synthesized circuit.
        The default is ``0``.

        Returns
        -------
        str
        """
        capacitor_ls_string = self._dll_interface.get_string(self._dll.getLumpedCapacitorLs)
        return capacitor_ls_string

    @capacitor_ls.setter
    def capacitor_ls(self, capacitor_ls_string):
        self._dll_interface.set_string(self._dll.setLumpedCapacitorLs, capacitor_ls_string)

    @property
    def inductor_q(self) -> str:
        """Q factor value of non-ideal inductors in the synthesized circuit.
        The default is ``infinite``.

        Returns
        -------
        str
        """
        inductor_q_string = self._dll_interface.get_string(self._dll.getLumpedInductorQ)
        return inductor_q_string

    @inductor_q.setter
    def inductor_q(self, inductor_q_string):
        self._dll_interface.set_string(self._dll.setLumpedInductorQ, inductor_q_string)

    @property
    def inductor_rs(self) -> str:
        """Series resistor value of non-ideal inductors in the synthesized circuit.
        The default is` ``0``.

        Returns
        -------
        str
        """
        inductor_rs_string = self._dll_interface.get_string(self._dll.getLumpedInductorRs)
        return inductor_rs_string

    @inductor_rs.setter
    def inductor_rs(self, inductor_rs_string):
        self._dll_interface.set_string(self._dll.setLumpedInductorRs, inductor_rs_string)

    @property
    def inductor_rp(self) -> str:
        """Shunt resistor value of non-ideal inductors in the synthesized circuit.
        The default is ``infinite``.

        Returns
        -------
        str
        """
        inductor_rp_string = self._dll_interface.get_string(self._dll.getLumpedInductorRp)
        return inductor_rp_string

    @inductor_rp.setter
    def inductor_rp(self, inductor_rp_string):
        self._dll_interface.set_string(self._dll.setLumpedInductorRp, inductor_rp_string)

    @property
    def inductor_cp(self) -> str:
        """Shunt capacitor value of non-ideal inductors in the synthesized circuit.
        The default is ``0``.

        Returns
        -------
        str
        """
        inductor_cp_string = self._dll_interface.get_string(self._dll.getLumpedInductorCp)
        return inductor_cp_string

    @inductor_cp.setter
    def inductor_cp(self, inductor_cp_string):
        self._dll_interface.set_string(self._dll.setLumpedInductorCp, inductor_cp_string)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import POINTER
from ctypes import byref
from ctypes import c_bool
from ctypes import c_char_p
from ctypes import c_int
from ctypes import create_string_buffer
from enum import Enum

import pyaedt


class TableFormat(Enum):
    """Enumeration of transmission zeros table.

    **Attributes:**

    - RATIO: Represents transmission zeros ratio table.
    - BANDWIDTH: Represents transmission zeros bandwidth table.
    """

    RATIO = 0
    BANDWIDTH = 1


class TransmissionZeros:
    """Manipulates access to ratio and bandwidth entries in the tranmsission zeros table.

    This class lets you to enter, edit, or remove ratio and bandwidth entries
    in the tranmsission zeros table.
    The table includes the ratio or bandwidth and the position of the element creating
    the transmission zero in the associated circuit.
    The position of the transmission zero is the position of the element in
    the associated circuit that creates the transmission zero.
    The position is defined automatically if no value is provided.
    """

    def __init__(self, table_format):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._define_transmission_zeros_dll_functions()
        self.table_format = table_format

    def _define_transmission_zeros_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.getTransmissionZerosTableRowCount.argtypes = [POINTER(c_int), c_bool]
        self._dll.getTransmissionZerosTableRowCount.restype = c_int

        self._dll.getTransmissionZerosTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_bool,
            c_int,
        ]
        self._dll.getTransmissionZerosTableRow.restype = c_int

        self._dll.updateTransmissionZerosTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.updateTransmissionZerosTableRow.restype = c_int

        self._dll.appendTransmissionZerosTableRow.argtypes = [
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.appendTransmissionZerosTableRow.restype = c_int

        self._dll.insertTransmissionZerosTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.insertTransmissionZerosTableRow.restype = c_int

        self._dll.removeTransmissionZerosTableRow.argtypes = [c_int, c_bool]
        self._dll.removeTransmissionZerosTableRow.restype = c_int

        self._dll.clearTransmissionZerosTableRow.argtype = c_bool
        self._dll.clearTransmissionZerosTableRow.restype = c_int

        self._dll.defaultPositionEnabled.argtype = c_bool
        self._dll.defaultPositionEnabled.restype = c_int

    def table_format_to_bool(self):
        """Check if the entry format of the transmission zeros tables is ratio.
        If ``False``, the entry format is bandwidth.

        Returns
        -------
        bool
        """
        if self.table_format.value == TableFormat.BANDWIDTH.value:
            return False
        else:
            return True

    @property
    def row_count(self) -> int:
        """Number of transmission zeros in the transmission zeros table.
        The default is ``2``.

        Returns
        -------
        int
        """
        table_row_count = c_int()
        status = self._dll.getTransmissionZerosTableRowCount(byref(table_row_count), self.table_format_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(table_row_count.value)

    def row(self, row_index):
        """Get the transmission zero ratio or bandwidth and the position of the element
        causing the transmission zero from a row in the transmission zeros table.

        Parameters
        ----------
        row_index: int
            Index of the row. Valid values range from ``0`` to ``9``, inclusive.

        Returns
        -------
        tuple
            The tuple contains two strings.The first is the transmission zero ratio or bandwidth value,
            and the second is the position of the element causing the transmission zero.
        """
        zero_value_buffer = create_string_buffer(100)
        position_value_buffer = create_string_buffer(100)
        status = self._dll.getTransmissionZerosTableRow(
            row_index,
            zero_value_buffer,
            position_value_buffer,
            self.table_format_to_bool(),
            100,
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        zero_value_string = zero_value_buffer.value.decode("utf-8")
        position_value_string = position_value_buffer.value.decode("utf-8")
        return zero_value_string, position_value_string

    def update_row(self, row_index, zero="", position=""):
        """Update the transmission zero ratio or bandwidth and its position
        for a row in the transmission zeros table.

        Parameters
        ----------
        row_index: int
            Index of the row. Valid values range from ``0`` to ``9``, inclusive.
        zero: str, optional
            New transmission zero ratio or bandwidth value to set.
            If no value is specified, the value remains unchanged.
        position: str, optional
            New position of the element causing the transmission zero in the circuit.
            If no value is specified, the value remains unchanged.
        """
        zero_bytes_value = bytes(zero, "ascii")
        position_bytes_value = bytes(position, "ascii")
        status = self._dll.updateTransmissionZerosTableRow(
            row_index,
            zero_bytes_value,
            position_bytes_value,
            self.table_format_to_bool(),
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def append_row(self, zero, position=""):
        """Append a new row that includes the ratio or bandwidth and position.

        Parameters
        ----------
        zero: str
            Transmission zero ratio or bandwidth value.
        position: str
            Position of the element creating transmission zero in the associated circuit.
        """
        zero_bytes_value = bytes(zero, "ascii")
        position_bytes_value = bytes(position, "ascii")
        status = self._dll.appendTransmissionZerosTableRow(
            zero_bytes_value, position_bytes_value, self.table_format_to_bool()
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def insert_row(self, row_index, zero, position=""):
        """Insert a new row that includes the ratio or bandwidth and the position.

        Parameters
        ----------
        row_index: int
            Index for the new row in the transmission zeros table. Valid values range from ``0`` to ``9``, inclusive.
        zero: str
            Transmission zero ratio or bandwidth value.
        position: str
            Position of the element creating transmission zero in the associated circuit.
        """
        zero_bytes_value = bytes(zero, "ascii")
        position_bytes_value = bytes(position, "ascii")
        status = self._dll.insertTransmissionZerosTableRow(
            row_index,
            zero_bytes_value,
            position_bytes_value,
            self.table_format_to_bool(),
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def remove_row(self, row_index):
        """Remove a row, including the ratio or bandwidth and the position.

        Parameters
        ----------
        row_index: int
            Row index in the transmission zeros table. Valid values range from ``0`` to ``9``, inclusive.
        """
        status = self._dll.removeTransmissionZerosTableRow(row_index, self.table_format_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def clear_row(self):
        """Clear all entries in the transmission zeros table."""
        status = self._dll.clearTransmissionZerosTableRow(self.table_format_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def restore_default_positions(self):
        """Restore default positions of transmissison zeros."""
        status = self._dll.defaultPositionEnabled(self.table_format_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys

from pyaedt.filtersolutions_core.dll_interface import DllInterface

# Store the current module in a variable for easy access and modification within the module itself.
_this = sys.modules[__name__]

# Initialize the internal DLL interface attribute to ``None``. This is set to an actual
# ``DllInterface`` instance when needed, implementing a lazy initialization pattern.
_this._internal_dll_interface = None


def _dll_interface(version=None) -> DllInterface:
    if _this._internal_dll_interface is None:
        _this._internal_dll_interface = DllInterface(show_gui=False, version=version)
    elif version is not None and version != _this._internal_dll_interface.version:
        raise Exception(
            "The requested version {} does not match with the previously defined version {}.".format(
                version, _this._internal_dll_interface.version
            )
        )

    return _this._internal_dll_interface


def api_version() -> str:
    return _dll_interface().api_version()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import POINTER
from ctypes import byref
from ctypes import c_bool
from ctypes import c_char_p
from ctypes import c_int

import pyaedt


class LumpedTopology:
    """Defines attributes and parameters of lumped filters.

    This class lets you construct all the necessary attributes for the ``LumpedDesign`` class.
    """

    def __init__(self):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._define_topology_dll_functions()

    def _define_topology_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.setLumpedGeneratorResistor.argtype = c_char_p
        self._dll.setLumpedGeneratorResistor.restype = c_int
        self._dll.getLumpedGeneratorResistor.argtypes = [c_char_p, c_int]
        self._dll.getLumpedGeneratorResistor.restype = c_int

        self._dll.setLumpedLoadResistor.argtype = c_char_p
        self._dll.setLumpedLoadResistor.restype = c_int
        self._dll.getLumpedLoadResistor.argtypes = [c_char_p, c_int]
        self._dll.getLumpedLoadResistor.restype = c_int

        self._dll.setLumpedCurrentSource.argtype = c_bool
        self._dll.setLumpedCurrentSource.restype = c_int
        self._dll.getLumpedCurrentSource.argtype = POINTER(c_bool)
        self._dll.getLumpedCurrentSource.restype = c_int

        self._dll.setLumpedFirstElementShunt.argtype = c_bool
        self._dll.setLumpedFirstElementShunt.restype = c_int
        self._dll.getLumpedFirstElementShunt.argtype = POINTER(c_bool)
        self._dll.getLumpedFirstElementShunt.restype = c_int

        self._dll.setLumpedBridgeT.argtype = c_bool
        self._dll.setLumpedBridgeT.restype = c_int
        self._dll.getLumpedBridgeT.argtype = POINTER(c_bool)
        self._dll.getLumpedBridgeT.restype = c_int

        self._dll.setLumpedBridgeTLow.argtype = c_bool
        self._dll.setLumpedBridgeTLow.restype = c_int
        self._dll.getLumpedBridgeTLow.argtype = POINTER(c_bool)
        self._dll.getLumpedBridgeTLow.restype = c_int

        self._dll.setLumpedBridgeTHigh.argtype = c_bool
        self._dll.setLumpedBridgeTHigh.restype = c_int
        self._dll.getLumpedBridgeTHigh.argtype = POINTER(c_bool)
        self._dll.getLumpedBridgeTHigh.restype = c_int

        self._dll.setLumpedEqualInductors.argtype = c_bool
        self._dll.setLumpedEqualInductors.restype = c_int
        self._dll.getLumpedEqualInductors.argtype = POINTER(c_bool)
        self._dll.getLumpedEqualInductors.restype = c_int

        self._dll.setLumpedEqualCapacitors.argtype = c_bool
        self._dll.setLumpedEqualCapacitors.restype = c_int
        self._dll.getLumpedEqualCapacitors.argtype = POINTER(c_bool)
        self._dll.getLumpedEqualCapacitors.restype = c_int

        self._dll.setLumpedEqualLegs.argtype = c_bool
        self._dll.setLumpedEqualLegs.restype = c_int
        self._dll.getLumpedEqualLegs.argtype = POINTER(c_bool)
        self._dll.getLumpedEqualLegs.restype = c_int

        self._dll.setLumpedHighLowPass.argtype = c_bool
        self._dll.setLumpedHighLowPass.restype = c_int
        self._dll.getLumpedHighLowPass.argtype = POINTER(c_bool)
        self._dll.getLumpedHighLowPass.restype = c_int

        self._dll.setLumpedHighLowPassMinInd.argtype = c_bool
        self._dll.setLumpedHighLowPassMinInd.restype = c_int
        self._dll.getLumpedHighLowPassMinInd.argtype = POINTER(c_bool)
        self._dll.getLumpedHighLowPassMinInd.restype = c_int

        self._dll.setLumpedZigZag.argtype = c_bool
        self._dll.setLumpedZigZag.restype = c_int
        self._dll.getLumpedZigZag.argtype = POINTER(c_bool)
        self._dll.getLumpedZigZag.restype = c_int

        self._dll.setLumpedMinInd.argtype = c_bool
        self._dll.setLumpedMinInd.restype = c_int
        self._dll.getLumpedMinInd.argtype = POINTER(c_bool)
        self._dll.getLumpedMinInd.restype = c_int

        self._dll.setLumpedMinCap.argtype = c_bool
        self._dll.setLumpedMinCap.restype = c_int
        self._dll.getLumpedMinCap.argtype = POINTER(c_bool)
        self._dll.getLumpedMinCap.restype = c_int

        self._dll.setLumpedSourceRes.argtype = c_bool
        self._dll.setLumpedSourceRes.restype = c_int
        self._dll.getLumpedSourceRes.argtype = POINTER(c_bool)
        self._dll.getLumpedSourceRes.restype = c_int

        self._dll.setLumpedTrapTopology.argtype = c_bool
        self._dll.setLumpedTrapTopology.restype = c_int
        self._dll.getLumpedTrapTopology.argtype = POINTER(c_bool)
        self._dll.getLumpedTrapTopology.restype = c_int

        self._dll.setLumpedNodeCapGround.argtype = c_bool
        self._dll.setLumpedNodeCapGround.restype = c_int
        self._dll.getLumpedNodeCapGround.argtype = POINTER(c_bool)
        self._dll.getLumpedNodeCapGround.restype = c_int

        self._dll.setLumpedMatchImpedance.argtype = c_bool
        self._dll.setLumpedMatchImpedance.restype = c_int
        self._dll.getLumpedMatchImpedance.argtype = POINTER(c_bool)
        self._dll.getLumpedMatchImpedance.restype = c_int

        self._dll.setLumpedComplexTermination.argtype = c_bool
        self._dll.setLumpedComplexTermination.restype = c_int
        self._dll.getLumpedComplexTermination.argtype = POINTER(c_bool)
        self._dll.getLumpedComplexTermination.restype = c_int

        self._dll.setLumpedComplexElementTuneEnabled.argtype = c_bool
        self._dll.setLumpedComplexElementTuneEnabled.restype = c_int
        self._dll.getLumpedComplexElementTuneEnabled.argtype = POINTER(c_bool)
        self._dll.getLumpedComplexElementTuneEnabled.restype = c_int

        self._dll.getLumpedCircuitResponseSize.argtype = POINTER(c_int)
        self._dll.getLumpedCircuitResponseSize.restype = c_int
        self._dll.getLumpedCircuitResponse.argtypes = [c_char_p, c_int]
        self._dll.getLumpedCircuitResponse.restype = c_int

    @property
    def generator_resistor(self) -> str:
        """Generator resistor. The default is ``50``.

        Returns
        -------
        str
        """
        generator_resistor_string = self._dll_interface.get_string(self._dll.getLumpedGeneratorResistor)
        return generator_resistor_string

    @generator_resistor.setter
    def generator_resistor(self, generator_resistor_string):
        self._dll_interface.set_string(self._dll.setLumpedGeneratorResistor, generator_resistor_string)

    @property
    def load_resistor(self) -> str:
        """Load resistor. The default is ``50``.

        Returns
        -------
        str
        """
        load_resistor_string = self._dll_interface.get_string(self._dll.getLumpedLoadResistor)
        return load_resistor_string

    @load_resistor.setter
    def load_resistor(self, load_resistor_string):
        self._dll_interface.set_string(self._dll.setLumpedLoadResistor, load_resistor_string)

    @property
    def current_source(self) -> bool:
        """Flag indicating if the current source in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        current_source = c_bool()
        status = self._dll.getLumpedCurrentSource(byref(current_source))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(current_source.value)

    @current_source.setter
    def current_source(self, current_source: bool):
        status = self._dll.setLumpedCurrentSource(current_source)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def first_shunt(self) -> bool:
        """Flag indicating if shunt elements are first in the synthesized circuit.
        If ``False``, series elements are first.

        Returns
        -------
        bool
        """
        first_shunt = c_bool()
        status = self._dll.getLumpedFirstElementShunt(byref(first_shunt))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(first_shunt.value)

    @first_shunt.setter
    def first_shunt(self, first_shunt: bool):
        status = self._dll.setLumpedFirstElementShunt(first_shunt)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def bridge_t(self) -> bool:
        """Flag indicating if the bridgeT topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        bridge_t = c_bool()
        status = self._dll.getLumpedBridgeT(byref(bridge_t))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(bridge_t.value)

    @bridge_t.setter
    def bridge_t(self, bridge_t: bool):
        status = self._dll.setLumpedBridgeT(bridge_t)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def bridge_t_low(self) -> bool:
        """Flag indicating if the bridgeT topology for the lower frequency band in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        bridge_t_low = c_bool()
        status = self._dll.getLumpedBridgeTLow(byref(bridge_t_low))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(bridge_t_low.value)

    @bridge_t_low.setter
    def bridge_t_low(self, bridge_t_low: bool):
        status = self._dll.setLumpedBridgeTLow(bridge_t_low)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def bridge_t_high(self) -> bool:
        """Flag indicating if the bridgeT topology for the higher frequency band in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        bridge_t_high = c_bool()
        status = self._dll.getLumpedBridgeTHigh(byref(bridge_t_high))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(bridge_t_high.value)

    @bridge_t_high.setter
    def bridge_t_high(self, bridge_t_high: bool):
        status = self._dll.setLumpedBridgeTHigh(bridge_t_high)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def equal_inductors(self) -> bool:
        """Flag indicating if the equal inductors topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        equal_inductors = c_bool()
        status = self._dll.getLumpedEqualInductors(byref(equal_inductors))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(equal_inductors.value)

    @equal_inductors.setter
    def equal_inductors(self, equal_inductors: bool):
        status = self._dll.setLumpedEqualInductors(equal_inductors)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def equal_capacitors(self) -> bool:
        """Flag indicating if the equal capacitors topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        equal_capacitors = c_bool()
        status = self._dll.getLumpedEqualCapacitors(byref(equal_capacitors))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(equal_capacitors.value)

    @equal_capacitors.setter
    def equal_capacitors(self, equal_capacitors: bool):
        status = self._dll.setLumpedEqualCapacitors(equal_capacitors)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def equal_legs(self) -> bool:
        """Flag indicating if the equal pairs shunt or series legs topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        equal_legs = c_bool()
        status = self._dll.getLumpedEqualLegs(byref(equal_legs))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(equal_legs.value)

    @equal_legs.setter
    def equal_legs(self, equal_legs: bool):
        status = self._dll.setLumpedEqualLegs(equal_legs)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def high_low_pass(self) -> bool:
        """Flag indicating if the high and low pass topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        high_low_pass = c_bool()
        status = self._dll.getLumpedHighLowPass(byref(high_low_pass))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(high_low_pass.value)

    @high_low_pass.setter
    def high_low_pass(self, high_low_pass: bool):
        status = self._dll.setLumpedHighLowPass(high_low_pass)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def high_low_pass_min_ind(self) -> bool:
        """Flag indicating if the high and low pass topology with minimum inductors
        in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        high_low_pass_min_ind = c_bool()
        status = self._dll.getLumpedHighLowPassMinInd(byref(high_low_pass_min_ind))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(high_low_pass_min_ind.value)

    @high_low_pass_min_ind.setter
    def high_low_pass_min_ind(self, high_low_pass_min_ind: bool):
        status = self._dll.setLumpedHighLowPassMinInd(high_low_pass_min_ind)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def zig_zag(self) -> bool:
        """Flag indicating if the zig-zag topology with minimum inductors in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        zig_zag = c_bool()
        status = self._dll.getLumpedZigZag(byref(zig_zag))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(zig_zag.value)

    @zig_zag.setter
    def zig_zag(self, zig_zag: bool):
        status = self._dll.setLumpedZigZag(zig_zag)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def min_ind(self) -> bool:
        """Flag indicating if the minimum inductors topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        min_ind = c_bool()
        status = self._dll.getLumpedMinInd(byref(min_ind))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(min_ind.value)

    @min_ind.setter
    def min_ind(self, min_ind: bool):
        status = self._dll.setLumpedMinInd(min_ind)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def min_cap(self) -> bool:
        """Flag indicating if the minimum capacitors topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        min_cap = c_bool()
        status = self._dll.getLumpedMinCap(byref(min_cap))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(min_cap.value)

    @min_cap.setter
    def min_cap(self, min_cap: bool):
        status = self._dll.setLumpedMinCap(min_cap)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def set_source_res(self) -> bool:
        """Flag indicating if the matched source resistor for zig-zag topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        set_source_res = c_bool()
        status = self._dll.getLumpedSourceRes(byref(set_source_res))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(set_source_res.value)

    @set_source_res.setter
    def set_source_res(self, set_source_res: bool):
        status = self._dll.setLumpedSourceRes(set_source_res)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def trap_topology(self) -> bool:
        """Flag indicating if the trap topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        trap_topology = c_bool()
        status = self._dll.getLumpedTrapTopology(byref(trap_topology))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(trap_topology.value)

    @trap_topology.setter
    def trap_topology(self, trap_topology: bool):
        status = self._dll.setLumpedTrapTopology(trap_topology)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def node_cap_ground(self) -> bool:
        """Flag indicating if the parasitic capacitors to ground topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        node_cap_ground = c_bool()
        status = self._dll.getLumpedNodeCapGround(byref(node_cap_ground))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(node_cap_ground.value)

    @node_cap_ground.setter
    def node_cap_ground(self, node_cap_ground: bool):
        status = self._dll.setLumpedNodeCapGround(node_cap_ground)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def match_impedance(self) -> bool:
        """Flag indicating if the automatic matched impedance topology in the synthesized circuit is enabled.

        Returns
        -------
        bool
        """
        match_impedance = c_bool()
        status = self._dll.getLumpedMatchImpedance(byref(match_impedance))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(match_impedance.value)

    @match_impedance.setter
    def match_impedance(self, match_impedance: bool):
        status = self._dll.setLumpedMatchImpedance(match_impedance)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def complex_termination(self) -> bool:
        """Flag indicating if the lumped filter complex termination is enabled.

        Returns
        -------
        bool
        """
        complex_termination = c_bool()
        status = self._dll.getLumpedComplexTermination(byref(complex_termination))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(complex_termination.value)

    @complex_termination.setter
    def complex_termination(self, complex_termination: bool):
        status = self._dll.setLumpedComplexTermination(complex_termination)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def complex_element_tune_enabled(self) -> bool:
        """Flag indicating if the element tune option is enabled.

        Returns
        -------
        bool
        """
        complex_element_tune_enabled = c_bool()
        status = self._dll.getLumpedComplexElementTuneEnabled(byref(complex_element_tune_enabled))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(complex_element_tune_enabled.value)

    @complex_element_tune_enabled.setter
    def complex_element_tune_enabled(self, complex_element_tune_enabled: bool):
        status = self._dll.setLumpedComplexElementTuneEnabled(complex_element_tune_enabled)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def circuit_response(self):
        """Execute real filter synthesis"""
        size = c_int()
        status = self._dll.getLumpedCircuitResponseSize(byref(size))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        circuit_response_string = self._dll_interface.get_string(
            self._dll.getLumpedCircuitResponse, max_size=size.value
        )
        return circuit_response_string



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import POINTER
from ctypes import byref
from ctypes import c_bool
from ctypes import c_char_p
from ctypes import c_int
from enum import Enum

import pyaedt


class FilterType(Enum):
    """Provides an enum of filter types with associated mathematical formulations.

    **Attributes:**

    - GAUSSIAN: Represents a Gaussian filter.
    - BESSEL: Represents a Bessel filter.
    - BUTTERWORTH: Represents a Butterworth filter.
    - LEGENDRE: Represents a Legendre filter.
    - CHEBYSHEV_I: Represents a Chevyshev type I filter.
    - CHEBYSHEV_II: Represents a Chevyshev type II filter.
    - HOURGLASS: Represents an hourglass filter.
    - ELLIPTIC: Represents an elliptic filter.

    Custom, raised-cos, and matched filter types are not available in this release.
    """

    GAUSSIAN = 0
    BESSEL = 1
    BUTTERWORTH = 2
    LEGENDRE = 3
    CHEBYSHEV_I = 4
    CHEBYSHEV_II = 5
    HOURGLASS = 6
    ELLIPTIC = 7
    DELAY = 8


#   CUSTOM = 8
#   RAISED_COS = 9
#   MATCHED = 10
#   DELAY = 11


class FilterClass(Enum):
    """Provides an enum of filter types for single-band and multiple-bands filters.

    **Attributes:**

    - LOW_PASS: Represents a low-pass filter.
    - HIGH_PASS: Represents a high-pass filter.
    - DIPLEXER_1: Represents a first group of diplexer filter.
    - BAND_PASS: Represents a band-pass filter.
    - BAND_STOP: Represents a band-stop filter.
    - DIPLEXER_2: Represents a second group of diplexer filter.
    - LOW_BAND: Represents a combined low-pass and multi-band filter.
    - BAND_HIGH: Represents a combined high-pass and multi-band filter.
    - BAND_BAND: Represents a multi-band pass filter.
    - STOP_STOP: Represents a multi-band stop filter.
    """

    LOW_PASS = 0
    HIGH_PASS = 1
    DIPLEXER_1 = 2
    BAND_PASS = 3
    BAND_STOP = 4
    DIPLEXER_2 = 5
    LOW_BAND = 6
    BAND_HIGH = 7
    BAND_BAND = 8
    STOP_STOP = 9


class FilterImplementation(Enum):
    """Provides an enum of filter implementation types.

    **Attributes:**

    - LUMPED: Represents a lumped implementation.
    - DISTRIB: Represents a distributed implementation.
    - ACTIVE: Represents an active implementation.
    - SWCAP: Represents a switched capacitor implementation.
    - DIGITAL: Represents a digital implementation.
    """

    LUMPED = 0
    DISTRIB = 1
    ACTIVE = 2
    SWCAP = 3
    DIGITAL = 4


class DiplexerType(Enum):
    """Provides an enum of diplexer and triplexer types.

    **Attributes:**

    - HI_LO: Represents a high-pass, low-pass diplexer type.
    - BP_1: Represents a band-pass, band-pass diplexer type.
    - BP_2: Represents a band-pass, band-pass diplexer type.
    - BP_BS: Represents a band-pass, band-stop diplexer type.
    - TRIPLEXER_1: Represents a low-pass, band-pass, and high-pass triplexer type.
    - TRIPLEXER_2: Represents a low-pass, band-pass, and high-pass triplexer type.
    """

    HI_LO = 0
    BP_1 = 1
    BP_2 = 2
    BP_BS = 3
    TRIPLEXER_1 = 4
    TRIPLEXER_2 = 5


class BesselRipplePercentage(Enum):
    """Provides an enum of peak-to-peak group delay ripple magnitudes as percents of averages for Bessel filters.

    **Attributes:**

    - ZERO: 0%
    - HALF: 0.5%
    - ONE: 1%
    - TWO: 2%
    - FIVE: 5%
    - TEN: 10%
    """

    ZERO = 0
    HALF = 1
    ONE = 2
    TWO = 3
    FIVE = 4
    TEN = 5


class GaussianTransition(Enum):
    """Provides an enum of transition attenuations in dB for Gaussian filters to improve group delay response.

    **Attributes:**

    - TRANSITION_NONE: 0dB
    - TRANSITION_3_DB: 3dB
    - TRANSITION_6_DB: 6dB
    - TRANSITION_9_DB: 9dB
    - TRANSITION_12_DB: 12dB
    - TRANSITION_15_DB: 15dB
    """

    TRANSITION_NONE = 0
    TRANSITION_3_DB = 1
    TRANSITION_6_DB = 2
    TRANSITION_9_DB = 3
    TRANSITION_12_DB = 4
    TRANSITION_15_DB = 5


class GaussianBesselReflection(Enum):
    """Provides an enum of synthesis methods for Gaussian and Bessel filters.

    **Attributes:**

    - OPTION_1: The first method for filter synthesis.
    - OPTION_2: The second method for filter synthesis.
    - OPTION_3: The third method for filter synthesis.
    """

    OPTION_1 = 0
    OPTION_2 = 1
    OPTION_3 = 2


class RippleConstrictionBandSelect(Enum):
    """Provides an enum of the bands to apply constrict the ripple parameter.

    **Attributes:**

    - STOP: Stop band
    - PASS: Pass band
    - BOTH: Stop and pass bands
    """

    STOP = 0
    PASS = 1
    BOTH = 2


class SinglePointRippleInfZeros(Enum):
    """Provides an enum for either one or three non-infinite zeros at the single frequency point to confine the ripple.

    **Attributes:**

    - RIPPLE_INF_ZEROS_1: One zero
    - RIPPLE_INF_ZEROS_3: Three zeros
    """

    RIPPLE_INF_ZEROS_1 = 0
    RIPPLE_INF_ZEROS_3 = 1


class PassbandDefinition(Enum):
    """Provides an enum to get either center frequency and bandwidth or corner frequencies.

    **Attributes:**

    - CENTER_FREQUENCY: Define the passband by the center frequency and bandwidth.
    - CORNER_FREQUENCIES: Define the passband by the corner frequencies.
    """

    CENTER_FREQUENCY = 0
    CORNER_FREQUENCIES = 1


class StopbandDefinition(Enum):
    """Provides an enum for comparing the stop band parameter to the pass band parameter.

    **Attributes:**

    - RATIO: Ratio between the stop band and pass band frequencies.
    - FREQUENCY: Explicit frequency.
    - ATTENUATION_DB: Attenuation in decibels.
    """

    RATIO = 0
    FREQUENCY = 1
    ATTENUATION_DB = 2


class Attributes:
    """Defines attributes and parameters of filters.

    This class allows you to construct all the necessary attributes for the ``FilterDesign`` class.
    """

    def __init__(self):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._dll_interface.restore_defaults()
        self._define_attributes_dll_functions()

    def _define_attributes_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.setFilterType.argtype = c_char_p
        self._dll.setFilterType.restype = c_int
        self._dll.getFilterType.argtypes = [c_char_p, c_int]
        self._dll.getFilterType.restype = c_int

        self._dll.setFilterClass.argtype = c_char_p
        self._dll.setFilterClass.restype = int
        self._dll.getFilterClass.argtypes = [c_char_p, c_int]
        self._dll.getFilterClass.restype = int

        self._dll.setFilterImplementation.argtype = c_char_p
        self._dll.setFilterImplementation.restype = c_int
        self._dll.getFilterImplementation.argtypes = [c_char_p, c_int]
        self._dll.getFilterImplementation.restype = c_int

        self._dll.setMultipleBandsEnabled.argtype = c_bool
        self._dll.setMultipleBandsEnabled.restype = c_int
        self._dll.getMultipleBandsEnabled.argtype = POINTER(c_bool)
        self._dll.getMultipleBandsEnabled.restype = c_int

        self._dll.setMultipleBandsLowPassFrequency.argtype = c_char_p
        self._dll.setMultipleBandsLowPassFrequency.restype = c_int
        self._dll.getMultipleBandsLowPassFrequency.argtypes = [c_char_p, c_int]
        self._dll.getMultipleBandsLowPassFrequency.restype = c_int

        self._dll.setMultipleBandsHighPassFrequency.argtype = c_char_p
        self._dll.setMultipleBandsHighPassFrequency.restype = c_int
        self._dll.getMultipleBandsHighPassFrequency.argtypes = [c_char_p, c_int]
        self._dll.getMultipleBandsHighPassFrequency.restype = c_int

        self._dll.setDiplexerType.argtype = c_char_p
        self._dll.setDiplexerType.restype = c_int
        self._dll.getDiplexerType.argtypes = [c_char_p, c_int]
        self._dll.getDiplexerType.restype = c_int

        self._dll.setDiplexerType.argtype = c_char_p
        self._dll.setDiplexerType.restype = c_int
        self._dll.getDiplexerType.argtypes = [c_char_p, c_int]
        self._dll.getDiplexerType.restype = c_int

        self._dll.setOrder.argtype = c_int
        self._dll.setOrder.restype = c_int
        self._dll.getOrder.argtype = POINTER(c_int)
        self._dll.getOrder.restype = c_int

        self._dll.setMinimumOrderStopbandAttenuationdB.argtype = c_char_p
        self._dll.setMinimumOrderStopbandAttenuationdB.restype = c_int
        self._dll.getMinimumOrderStopbandAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getMinimumOrderStopbandAttenuationdB.restype = c_int

        self._dll.setMinimumOrderStopbandFrequency.argtype = c_char_p
        self._dll.setMinimumOrderStopbandFrequency.restype = c_int
        self._dll.getMinimumOrderStopbandFrequency.argtypes = [c_char_p, c_int]
        self._dll.getMinimumOrderStopbandFrequency.restype = c_int

        self._dll.setMinimumOrderGroupDelayError.argtype = c_char_p
        self._dll.setMinimumOrderGroupDelayError.restype = c_int
        self._dll.getMinimumOrderGroupDelayError.argtypes = [c_char_p, c_int]
        self._dll.getMinimumOrderGroupDelayError.restype = c_int

        self._dll.setMinimumOrderGroupDelayCutoff.argtype = c_char_p
        self._dll.setMinimumOrderGroupDelayCutoff.restype = c_int
        self._dll.getMinimumOrderGroupDelayCutoff.argtypes = [c_char_p, c_int]
        self._dll.getMinimumOrderGroupDelayCutoff.restype = c_int

        self._dll.setIdealMinimumOrder.argtype = POINTER(c_int)
        self._dll.setIdealMinimumOrder.restype = c_int

        self._dll.getErrorMessage.argtypes = [c_char_p, c_int]
        self._dll.getErrorMessage.restype = c_int

        self._dll.setPassbandDef.argtype = c_int
        self._dll.setPassbandDef.restype = c_int
        self._dll.getPassbandDef.argtype = POINTER(c_int)
        self._dll.getPassbandDef.restype = c_int

        self._dll.setCenterFrequency.argtype = c_char_p
        self._dll.setCenterFrequency.restype = c_int
        self._dll.getCenterFrequency.argtypes = [c_char_p, c_int]
        self._dll.getCenterFrequency.restype = c_int

        self._dll.setDelayTime.argtype = c_char_p
        self._dll.setDelayTime.restype = c_int
        self._dll.getDelayTime.argtypes = [c_char_p, c_int]
        self._dll.getDelayTime.restype = c_int

        self._dll.setPassbandFrequency.argtype = c_char_p
        self._dll.setPassbandFrequency.restype = c_int
        self._dll.getPassbandFrequency.argtypes = [c_char_p, c_int]
        self._dll.getPassbandFrequency.restype = c_int

        self._dll.setLowerFrequency.argtype = c_char_p
        self._dll.setLowerFrequency.restype = c_int
        self._dll.getLowerFrequency.argtypes = [c_char_p, c_int]
        self._dll.getLowerFrequency.restype = c_int

        self._dll.setUpperFrequency.argtype = c_char_p
        self._dll.setUpperFrequency.restype = c_int
        self._dll.getUpperFrequency.argtypes = [c_char_p, c_int]
        self._dll.getUpperFrequency.restype = c_int

        self._dll.setStopbandDef.argtype = c_int
        self._dll.setStopbandDef.restype = c_int
        self._dll.getStopbandDef.argtype = POINTER(c_int)
        self._dll.getStopbandDef.restype = c_int

        self._dll.setStopbandRatio.argtype = c_char_p
        self._dll.setStopbandRatio.restype = c_int
        self._dll.getStopbandRatio.argtypes = [c_char_p, c_int]
        self._dll.getStopbandRatio.restype = c_int

        self._dll.setStopbandFrequency.argtype = c_char_p
        self._dll.setStopbandFrequency.restype = c_int
        self._dll.getStopbandFrequency.argtypes = [c_char_p, c_int]
        self._dll.getStopbandFrequency.restype = c_int

        self._dll.setStopbandAttenuationdB.argtype = c_char_p
        self._dll.setStopbandAttenuationdB.restype = c_int
        self._dll.getStopbandAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getStopbandAttenuationdB.restype = c_int

        self._dll.setStandardCutoffEnabled.argtype = c_bool
        self._dll.setStandardCutoffEnabled.restype = c_int
        self._dll.getStandardCutoffEnabled.argtype = POINTER(c_bool)
        self._dll.getStandardCutoffEnabled.restype = c_int

        self._dll.setEquirippleDelayEnabled.argtype = c_bool
        self._dll.setEquirippleDelayEnabled.restype = c_int
        self._dll.getEquirippleDelayEnabled.argtype = POINTER(c_bool)
        self._dll.getEquirippleDelayEnabled.restype = c_int

        self._dll.setDelayRipplePeriod.argtype = c_char_p
        self._dll.setDelayRipplePeriod.restype = c_int
        self._dll.getDelayRipplePeriod.argtypes = [c_char_p, c_int]
        self._dll.getDelayRipplePeriod.restype = c_int

        self._dll.setGroupDealyRipplePercentage.argtype = c_int
        self._dll.setGroupDealyRipplePercentage.restype = c_int
        self._dll.setGroupDealyRipplePercentage.argtype = POINTER(c_int)
        self._dll.setGroupDealyRipplePercentage.restype = c_int

        self._dll.setCutoffAttenuationdB.argtype = c_char_p
        self._dll.setCutoffAttenuationdB.restype = c_int
        self._dll.getCutoffAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getCutoffAttenuationdB.restype = c_int

        self._dll.setBesselNormalizedDelayEnabled.argtype = c_bool
        self._dll.setBesselNormalizedDelayEnabled.restype = c_int
        self._dll.getBesselNormalizedDelayEnabled.argtype = POINTER(c_bool)
        self._dll.getBesselNormalizedDelayEnabled.restype = c_int

        self._dll.setBesselEquiRippleDelayPeriod.argtype = c_char_p
        self._dll.setBesselEquiRippleDelayPeriod.restype = c_int
        self._dll.getBesselEquiRippleDelayPeriod.argtypes = [c_char_p, c_int]
        self._dll.getBesselEquiRippleDelayPeriod.restype = c_int

        self._dll.setBesselRipplePercentage.argtype = c_int
        self._dll.setBesselRipplePercentage.restype = c_int
        self._dll.getBesselRipplePercentage.argtype = POINTER(c_int)
        self._dll.getBesselRipplePercentage.restype = c_int

        self._dll.setPassbandRipple.argtype = c_char_p
        self._dll.setPassbandRipple.restype = c_int
        self._dll.getPassbandRipple.argtypes = [c_char_p, c_int]
        self._dll.getPassbandRipple.restype = c_int

        self._dll.setArithSymmetry.argtype = c_bool
        self._dll.setArithSymmetry.restype = c_int
        self._dll.getArithSymmetry.argtype = POINTER(c_bool)
        self._dll.getArithSymmetry.restype = c_int

        self._dll.setAsymmetric.argtype = c_bool
        self._dll.setAsymmetric.restype = c_int
        self._dll.getAsymmetric.argtype = POINTER(c_bool)
        self._dll.getAsymmetric.restype = c_int

        self._dll.setAsymmetricLowOrder.argtype = c_int
        self._dll.setAsymmetricLowOrder.restype = c_int
        self._dll.getAsymmetricLowOrder.argtype = POINTER(c_int)
        self._dll.getAsymmetricLowOrder.restype = c_int

        self._dll.setAsymmetricHighOrder.argtype = c_int
        self._dll.setAsymmetricHighOrder.restype = c_int
        self._dll.getAsymmetricHighOrder.argtype = POINTER(c_int)
        self._dll.getAsymmetricHighOrder.restype = c_int

        self._dll.setAsymmetricLowStopbandRatio.argtype = c_char_p
        self._dll.setAsymmetricLowStopbandRatio.restype = c_int
        self._dll.getAsymmetricLowStopbandRatio.argtypes = [c_char_p, c_int]
        self._dll.getAsymmetricLowStopbandRatio.restype = c_int

        self._dll.setAsymmetricHighStopbandRatio.argtype = c_char_p
        self._dll.setAsymmetricHighStopbandRatio.restype = c_int
        self._dll.getAsymmetricHighStopbandRatio.argtypes = [c_char_p, c_int]
        self._dll.getAsymmetricHighStopbandRatio.restype = c_int

        self._dll.setAsymmetricLowStopbandAttenuationdB.argtype = c_char_p
        self._dll.setAsymmetricLowStopbandAttenuationdB.restype = c_int
        self._dll.getAsymmetricLowStopbandAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getAsymmetricLowStopbandAttenuationdB.restype = c_int

        self._dll.setAsymmetricHighStopbandAttenuationdB.argtype = c_char_p
        self._dll.setAsymmetricHighStopbandAttenuationdB.restype = c_int
        self._dll.getAsymmetricHighStopbandAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getAsymmetricHighStopbandAttenuationdB.restype = c_int

        self._dll.setGaussianTransition.argtype = c_char_p
        self._dll.setGaussianTransition.restype = c_int
        self._dll.getGaussianTransition.argtypes = [c_char_p, c_int]
        self._dll.getGaussianTransition.restype = c_int

        self._dll.setGaussianBesselReflection.argtype = c_int
        self._dll.setGaussianBesselReflection.restype = c_int
        self._dll.getGaussianBesselReflection.argtype = POINTER(c_int)
        self._dll.getGaussianBesselReflection.restype = c_int

        self._dll.setEvenOrderMode.argtype = c_bool
        self._dll.setEvenOrderMode.restype = c_int
        self._dll.getEvenOrderMode.argtype = POINTER(c_bool)
        self._dll.getEvenOrderMode.restype = c_int

        self._dll.setEvenReflZeroTo0.argtype = c_bool
        self._dll.setEvenReflZeroTo0.restype = c_int
        self._dll.getEvenReflZeroTo0.argtype = POINTER(c_bool)
        self._dll.getEvenReflZeroTo0.restype = c_int

        self._dll.setEvenTrnZeroToInf.argtype = c_bool
        self._dll.setEvenTrnZeroToInf.restype = c_int
        self._dll.getEvenTrnZeroToInf.argtype = POINTER(c_bool)
        self._dll.getEvenTrnZeroToInf.restype = c_int

        self._dll.setConstrictRipple.argtype = c_bool
        self._dll.setConstrictRipple.restype = c_int
        self._dll.getConstrictRipple.argtype = POINTER(c_bool)
        self._dll.getConstrictRipple.restype = c_int

        self._dll.setSinglePointRipple.argtype = c_bool
        self._dll.setSinglePointRipple.restype = c_int
        self._dll.getSinglePointRipple.argtype = POINTER(c_bool)
        self._dll.getSinglePointRipple.restype = c_int

        self._dll.setHalfBandRipple.argtype = c_bool
        self._dll.setHalfBandRipple.restype = c_int
        self._dll.getHalfBandRipple.argtype = POINTER(c_bool)
        self._dll.getHalfBandRipple.restype = c_int

        self._dll.setRippleConstrictionPercent.argtype = c_char_p
        self._dll.setRippleConstrictionPercent.restype = c_int
        self._dll.getRippleConstrictionPercent.argtypes = [c_char_p, c_int]
        self._dll.getRippleConstrictionPercent.restype = c_int

        self._dll.setRippleConstrictionBandSelect.argtype = c_char_p
        self._dll.setRippleConstrictionBandSelect.restype = c_int
        self._dll.getRippleConstrictionBandSelect.argtypes = [c_char_p, c_int]
        self._dll.getRippleConstrictionBandSelect.restype = c_int

        self._dll.setSinglePointRippleNoninfiniteZeros.argtype = c_char_p
        self._dll.setSinglePointRippleNoninfiniteZeros.restype = c_int
        self._dll.getSinglePointRippleNoninfiniteZeros.argtypes = [c_char_p, c_int]
        self._dll.getSinglePointRippleNoninfiniteZeros.restype = c_int

        self._dll.setDelayEqualizer.argtype = c_bool
        self._dll.setDelayEqualizer.restype = c_int
        self._dll.getDelayEqualizer.argtype = POINTER(c_bool)
        self._dll.getDelayEqualizer.restype = c_int

        self._dll.setDelayEqualizerOrder.argtype = c_int
        self._dll.setDelayEqualizerOrder.restype = c_int
        self._dll.getDelayEqualizerOrder.argtype = POINTER(c_int)
        self._dll.getDelayEqualizerOrder.restype = c_int

        self._dll.setStandardDelayEquCut.argtype = c_bool
        self._dll.setStandardDelayEquCut.restype = c_int
        self._dll.getStandardDelayEquCut.argtype = POINTER(c_bool)
        self._dll.getStandardDelayEquCut.restype = c_int

        self._dll.setDelayEquCutoffAttenuationdB.argtype = c_char_p
        self._dll.setDelayEquCutoffAttenuationdB.restype = c_int
        self._dll.getDelayEquCutoffAttenuationdB.argtypes = [c_char_p, c_int]
        self._dll.getDelayEquCutoffAttenuationdB.restype = c_int

    @property
    def filter_type(self) -> FilterType:
        """Type (mathematical formulation) of the filter. The default is ``BUTTERWORTH``.
        The ``FilterType`` enum provides a list of all types.

        Returns
        -------
        :enum:`FilterType`
        """
        type_string = self._dll_interface.get_string(self._dll.getFilterType)
        return self._dll_interface.string_to_enum(FilterType, type_string)

    @filter_type.setter
    def filter_type(self, filter_type: FilterType):
        if filter_type:
            string_value = self._dll_interface.enum_to_string(filter_type)
            self._dll_interface.set_string(self._dll.setFilterType, string_value)

    @property
    def filter_class(self) -> FilterClass:
        """Class (band definition) of the filter. The default is ``LOW_PASS``.
        The ``FilterClass`` enum provides a list of all classes.

        Returns
        -------
        :enum:`FilterClass`
        """
        type_string = self._dll_interface.get_string(self._dll.getFilterClass)
        return self._dll_interface.string_to_enum(FilterClass, type_string)

    @filter_class.setter
    def filter_class(self, filter_class: FilterClass):
        if filter_class:
            string_value = self._dll_interface.enum_to_string(filter_class)
            self._dll_interface.set_string(self._dll.setFilterClass, string_value)

    @property
    def filter_implementation(self) -> FilterImplementation:
        """Technology for implementing the filter. The default is ``LUMPED``.
        The ``FilterImplementation`` enum provides a list of all implementations.

        Returns
        -------
        :enum:`FilterImplementation`
        """
        type_string = self._dll_interface.get_string(self._dll.getFilterImplementation)
        return self._dll_interface.string_to_enum(FilterImplementation, type_string)

    @filter_implementation.setter
    def filter_implementation(self, filter_implementation: FilterImplementation):
        if filter_implementation:
            string_value = self._dll_interface.enum_to_string(filter_implementation)
            self._dll_interface.set_string(self._dll.setFilterImplementation, string_value)

    @property
    def diplexer_type(self) -> DiplexerType:
        """Type of diplexer topology. This property is only applicable to lumped filters.

        - The default is ``HI_LO`` for the ``DIPLEXER_1`` filter class.

        - The default is ``BP_BS`` for the ``DIPLEXER_2`` filter class.

        The ``DiplexerType`` enum provides a full list of diplexer types.

        Returns
        -------
        :enum:`DiplexerType`
        """
        type_string = self._dll_interface.get_string(self._dll.getDiplexerType)
        return self._dll_interface.string_to_enum(DiplexerType, type_string)

    @diplexer_type.setter
    def diplexer_type(self, diplexer_type: DiplexerType):
        string_value = self._dll_interface.enum_to_string(diplexer_type)
        self._dll_interface.set_string(self._dll.setDiplexerType, string_value)

    @property
    def filter_multiple_bands_enabled(self) -> bool:
        """Flag indicating if the multiple bands table is enabled.

        Returns
        -------
        bool
        """
        filter_multiple_bands_enabled = c_bool()
        status = self._dll.getMultipleBandsEnabled(byref(filter_multiple_bands_enabled))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(filter_multiple_bands_enabled.value)

    @filter_multiple_bands_enabled.setter
    def filter_multiple_bands_enabled(self, filter_multiple_bands_enabled: bool):
        status = self._dll.setMultipleBandsEnabled(filter_multiple_bands_enabled)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def filter_multiple_bands_low_pass_frequency(self) -> str:
        """Multiple bands low-pass frequency of combined low-pass and band-pass filters. The default is ``1GHz``.

        Returns
        -------
        str
        """
        filter_multiple_bands_low_pass_freq_string = self._dll_interface.get_string(
            self._dll.getMultipleBandsLowPassFrequency
        )
        return filter_multiple_bands_low_pass_freq_string

    @filter_multiple_bands_low_pass_frequency.setter
    def filter_multiple_bands_low_pass_frequency(self, filter_multiple_bands_low_pass_freq_string):
        self._dll_interface.set_string(
            self._dll.setMultipleBandsLowPassFrequency,
            filter_multiple_bands_low_pass_freq_string,
        )

    @property
    def filter_multiple_bands_high_pass_frequency(self) -> str:
        """Multiple bands high-pass frequency of combined high-pass and band-pass filters. The default is ``1GHz``.

        Returns
        -------
        str
        """
        filter_multiple_bands_high_pass_freq_string = self._dll_interface.get_string(
            self._dll.getMultipleBandsHighPassFrequency
        )
        return filter_multiple_bands_high_pass_freq_string

    @filter_multiple_bands_high_pass_frequency.setter
    def filter_multiple_bands_high_pass_frequency(self, filter_multiple_bands_high_pass_freq_string):
        self._dll_interface.set_string(
            self._dll.setMultipleBandsHighPassFrequency,
            filter_multiple_bands_high_pass_freq_string,
        )

    @property
    def order(self) -> int:
        """Order of the filter. The default is ``5``.

        Returns
        -------
        int
        """
        order = c_int()
        status = self._dll.getOrder(byref(order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(order.value)

    @order.setter
    def order(self, order: int):
        status = self._dll.setOrder(order)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def minimum_order_stop_band_attenuation_db(self) -> str:
        """Filter stop band attenuation in dB for calculation of the filter minimum order.
        The default is ``50``.

        Returns
        -------
        str
        """
        minimum_order_stop_band_attenuation_db_string = self._dll_interface.get_string(
            self._dll.getMinimumOrderStopbandAttenuationdB
        )
        return minimum_order_stop_band_attenuation_db_string

    @minimum_order_stop_band_attenuation_db.setter
    def minimum_order_stop_band_attenuation_db(self, minimum_order_stop_band_attenuation_db_string):
        self._dll_interface.set_string(
            self._dll.setMinimumOrderStopbandAttenuationdB,
            minimum_order_stop_band_attenuation_db_string,
        )

    @property
    def minimum_order_stop_band_frequency(self) -> str:
        """Filter stop band frequency for calculation of the filter minimum order.
        The default is ``10 GHz``.

        Returns
        -------
        str
        """
        minimum_order_stop_band_frequency_string = self._dll_interface.get_string(
            self._dll.getMinimumOrderStopbandFrequency
        )
        return minimum_order_stop_band_frequency_string

    @minimum_order_stop_band_frequency.setter
    def minimum_order_stop_band_frequency(self, minimum_order_stop_band_frequency_string):
        self._dll_interface.set_string(
            self._dll.setMinimumOrderStopbandFrequency,
            minimum_order_stop_band_frequency_string,
        )

    @property
    def minimum_order_group_delay_error_percent(self) -> str:
        """Filter maximum group delay in % for calculation of the filter minimum order.
        The default is ``5``.

        Returns
        -------
        str
        """
        minimum_order_group_delay_error_percent_string = self._dll_interface.get_string(
            self._dll.getMinimumOrderGroupDelayError
        )
        return minimum_order_group_delay_error_percent_string

    @minimum_order_group_delay_error_percent.setter
    def minimum_order_group_delay_error_percent(self, minimum_order_group_delay_error_percent):
        self._dll_interface.set_string(
            self._dll.setMinimumOrderGroupDelayError,
            minimum_order_group_delay_error_percent,
        )

    @property
    def minimum_order_group_delay_cutoff(self) -> str:
        """Filter group delay cutoff frequency for calculation of the filter minimum order.
        The default is ``10 GHz``.

        Returns
        -------
        str
        """
        minimum_order_group_delay_cutoff_string = self._dll_interface.get_string(
            self._dll.getMinimumOrderGroupDelayCutoff
        )
        return minimum_order_group_delay_cutoff_string

    @minimum_order_group_delay_cutoff.setter
    def minimum_order_group_delay_cutoff(self, minimum_order_group_delay_cutoff_string):
        self._dll_interface.set_string(
            self._dll.setMinimumOrderGroupDelayCutoff,
            minimum_order_group_delay_cutoff_string,
        )

    @property
    def ideal_minimum_order(self) -> int:
        """Filter minimum order for the defined stop band frequency and attenuation parameters.

        Returns
        -------
        int
        """
        minimum_order = c_int()
        status = self._dll.setIdealMinimumOrder(byref(minimum_order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(minimum_order.value)

    @property
    def delay_time(self) -> str:
        """Filter delay time.
        The default is ``1 ns``.

        Returns
        -------
        str
        """
        delay_time_string = self._dll_interface.get_string(self._dll.getDelayTime)
        return delay_time_string

    @delay_time.setter
    def delay_time(self, delay_time_string):
        self._dll_interface.set_string(self._dll.setDelayTime, delay_time_string)

    @property
    def pass_band_definition(self) -> PassbandDefinition:
        """Pass band frequency entry options.
        The default is ``CENTER_FREQUENCY``.

        Returns
        -------
        :enum:`PassbandDefinition`
        """
        index = c_int()
        pass_band_definition = list(PassbandDefinition)
        status = self._dll.getPassbandDef(byref(index))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        pass_band_definition = pass_band_definition[index.value]
        return pass_band_definition

    @pass_band_definition.setter
    def pass_band_definition(self, column: PassbandDefinition):
        status = self._dll.setPassbandDef(column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def pass_band_center_frequency(self) -> str:
        """Filter pass band or center frequency.
        The default is ``1 GHz``.

        Returns
        -------
        str
        """
        center_freq_string = self._dll_interface.get_string(self._dll.getCenterFrequency)
        return center_freq_string

    @pass_band_center_frequency.setter
    def pass_band_center_frequency(self, center_freq_string):
        self._dll_interface.set_string(self._dll.setCenterFrequency, center_freq_string)

    @property
    def pass_band_width_frequency(self) -> str:
        """Pass band width frequency for band pass or band stop filters.
        The default is ``200 MHz``.

        Returns
        -------
        str
        """
        pass_band_freq_string = self._dll_interface.get_string(self._dll.getPassbandFrequency)
        return pass_band_freq_string

    @pass_band_width_frequency.setter
    def pass_band_width_frequency(self, pass_band_freq_string):
        self._dll_interface.set_string(self._dll.setPassbandFrequency, pass_band_freq_string)

    @property
    def lower_frequency(self) -> str:
        """Filter lower corner frequency.
        The default is ``905 MHz``.

        Returns
        -------
        str
        """
        lower_freq_string = self._dll_interface.get_string(self._dll.getLowerFrequency)
        return lower_freq_string

    @lower_frequency.setter
    def lower_frequency(self, lower_freq_string):
        self._dll_interface.set_string(self._dll.setLowerFrequency, lower_freq_string)

    @property
    def upper_frequency(self) -> str:
        """Filter upper corner frequency.
        The default is ``1.105 MHz``.

        Returns
        -------
        str
        """
        upper_freq_string = self._dll_interface.get_string(self._dll.getUpperFrequency)
        return upper_freq_string

    @upper_frequency.setter
    def upper_frequency(self, upper_freq_string):
        self._dll_interface.set_string(self._dll.setUpperFrequency, upper_freq_string)

    @property
    def stop_band_definition(self) -> StopbandDefinition:
        """Stop band parameter entry option.
        The default is ``RATIO``.

        Returns
        -------
        :enum:`StopbandDefinition`
        """
        index = c_int()
        stop_band_definition = list(StopbandDefinition)
        status = self._dll.getStopbandDef(byref(index))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        stop_band_definition = stop_band_definition[index.value]
        return stop_band_definition

    @stop_band_definition.setter
    def stop_band_definition(self, column: StopbandDefinition):
        status = self._dll.setStopbandDef(column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def stop_band_ratio(self) -> str:
        """Filter stop band ratio.
        The default is ``1.2``.

        Returns
        -------
        str
        """
        stop_band_ratio_string = self._dll_interface.get_string(self._dll.getStopbandRatio)
        return stop_band_ratio_string

    @stop_band_ratio.setter
    def stop_band_ratio(self, stop_band_ratio_string):
        self._dll_interface.set_string(self._dll.setStopbandRatio, stop_band_ratio_string)

    @property
    def stop_band_frequency(self) -> str:
        """Filter stop band frequency.
        The default is ``1.2 GHz``.

        Returns
        -------
        str
        """
        stop_band_frequency_string = self._dll_interface.get_string(self._dll.getStopbandFrequency)
        return stop_band_frequency_string

    @stop_band_frequency.setter
    def stop_band_frequency(self, stop_band_frequency_string):
        self._dll_interface.set_string(self._dll.setStopbandFrequency, stop_band_frequency_string)

    @property
    def stop_band_attenuation_db(self) -> str:
        """Filter stop band attenuation in dB.
        The default is ``60 dB``.

        Returns
        -------
        str
        """
        stop_band_attenuation_db_string = self._dll_interface.get_string(self._dll.getStopbandAttenuationdB)
        return stop_band_attenuation_db_string

    @stop_band_attenuation_db.setter
    def stop_band_attenuation_db(self, stop_band_attenuation_db_string):
        self._dll_interface.set_string(self._dll.setStopbandAttenuationdB, stop_band_attenuation_db_string)

    @property
    def standard_pass_band_attenuation(self) -> bool:
        """Flag indicating if the standard cut is enabled.

        Returns
        -------
        bool
        """
        standard_pass_band_attenuation = c_bool()
        status = self._dll.getStandardCutoffEnabled(byref(standard_pass_band_attenuation))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(standard_pass_band_attenuation.value)

    @standard_pass_band_attenuation.setter
    def standard_pass_band_attenuation(self, standard_pass_band_attenuation: bool):
        status = self._dll.setStandardCutoffEnabled(standard_pass_band_attenuation)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def equiripple_delay(self) -> bool:
        """Flag indicating if the equiripple delay is enabled.

        Returns
        -------
        bool
        """
        equiripple_delay = c_bool()
        status = self._dll.getEquirippleDelayEnabled(byref(equiripple_delay))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(equiripple_delay.value)

    @equiripple_delay.setter
    def equiripple_delay(self, equiripple_delay: bool):
        status = self._dll.setEquirippleDelayEnabled(equiripple_delay)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def group_delay_ripple_period(self) -> str:
        """Filter approximate normalized group delay ripple period.
        The default is ''2''.

        Returns
        -------
        str
        """
        group_delay_ripple_period_string = self._dll_interface.get_string(self._dll.getDelayRipplePeriod)
        return group_delay_ripple_period_string

    @group_delay_ripple_period.setter
    def group_delay_ripple_period(self, group_delay_ripple_period_string):
        self._dll_interface.set_string(
            self._dll.setDelayRipplePeriod,
            group_delay_ripple_period_string,
        )

    @property
    def normalized_group_delay_percentage(self) -> int:
        """Bessel filter ripple percentage.
        The default is ''0''.

        Returns
        -------
        int
        """
        index = c_int()
        normalized_group_delay_percentage = list(BesselRipplePercentage)
        status = self._dll.getGroupDealyRipplePercentage(byref(index))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        normalized_group_delay_percentage_string = normalized_group_delay_percentage[index.value]
        return normalized_group_delay_percentage_string

    @normalized_group_delay_percentage.setter
    def normalized_group_delay_percentage(self, column: BesselRipplePercentage):
        status = self._dll.setGroupDealyRipplePercentage(column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def standard_pass_band_attenuation_value_db(self) -> str:
        """Filter cut off attenuation in dB.
        The default is ''3.01 dB''.

        Returns
        -------
        str
        """
        standard_pass_band_attenuation_value_db_string = self._dll_interface.get_string(
            self._dll.getCutoffAttenuationdB
        )
        return standard_pass_band_attenuation_value_db_string

    @standard_pass_band_attenuation_value_db.setter
    def standard_pass_band_attenuation_value_db(self, standard_pass_band_attenuation_value_db_string):
        self._dll_interface.set_string(
            self._dll.setCutoffAttenuationdB,
            standard_pass_band_attenuation_value_db_string,
        )

    @property
    def bessel_normalized_delay(self) -> bool:
        """Flag indicating if the normalized delay is enabled.

        Returns
        -------
        bool
        """
        bessel_normalized_delay = c_bool()
        status = self._dll.getBesselNormalizedDelayEnabled(byref(bessel_normalized_delay))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(bessel_normalized_delay.value)

    @bessel_normalized_delay.setter
    def bessel_normalized_delay(self, bessel_normalized_delay: bool):
        status = self._dll.setBesselNormalizedDelayEnabled(bessel_normalized_delay)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def bessel_normalized_delay_period(self) -> str:
        """Bessel filter normalized delay period.
        The default is ''2''.

        Returns
        -------
        str
        """
        bessel_normalized_delay_period_string = self._dll_interface.get_string(self._dll.getBesselEquiRippleDelayPeriod)
        return bessel_normalized_delay_period_string

    @bessel_normalized_delay_period.setter
    def bessel_normalized_delay_period(self, bessel_normalized_delay_period_string):
        self._dll_interface.set_string(
            self._dll.setBesselEquiRippleDelayPeriod,
            bessel_normalized_delay_period_string,
        )

    @property
    def bessel_normalized_delay_percentage(self) -> int:
        """Bessel filter ripple percentage.
        The default is ''0''.

        Returns
        -------
        int
        """
        index = c_int()
        bessel_normalized_delay_percentage = list(BesselRipplePercentage)
        status = self._dll.getBesselRipplePercentage(byref(index))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        bessel_normalized_delay_percentage_string = bessel_normalized_delay_percentage[index.value]
        return bessel_normalized_delay_percentage_string

    @bessel_normalized_delay_percentage.setter
    def bessel_normalized_delay_percentage(self, column: BesselRipplePercentage):
        status = self._dll.setBesselRipplePercentage(column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def pass_band_ripple(self) -> str:
        """Filter pass band ripple in dB.
        The default is ''0.05 dB''.

        Returns
        -------
        str
        """
        pass_band_ripple_string = self._dll_interface.get_string(self._dll.getPassbandRipple)
        return pass_band_ripple_string

    @pass_band_ripple.setter
    def pass_band_ripple(self, pass_band_ripple_string):
        self._dll_interface.set_string(self._dll.setPassbandRipple, pass_band_ripple_string)

    @property
    def arith_symmetry(self) -> bool:
        """Flag indicating if the arithmetic symmetry is enabled.

        Returns
        -------
        bool
        """
        arith_symmetry = c_bool()
        status = self._dll.getArithSymmetry(byref(arith_symmetry))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(arith_symmetry.value)

    @arith_symmetry.setter
    def arith_symmetry(self, arith_symmetry: bool):
        status = self._dll.setArithSymmetry(arith_symmetry)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def asymmetric(self) -> bool:
        """Flag indicating if the asymmetric is enabled.

        Returns
        -------
        bool
        """
        asymmetric = c_bool()
        status = self._dll.getAsymmetric(byref(asymmetric))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(asymmetric.value)

    @asymmetric.setter
    def asymmetric(self, asymmetric: bool):
        status = self._dll.setAsymmetric(asymmetric)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def asymmetric_low_order(self) -> int:
        """Order for low side of an asymmetric filter.
        The default is ''5''.

        Returns
        -------
        int
        """
        asymmetric_low_order = c_int()
        status = self._dll.getAsymmetricLowOrder(byref(asymmetric_low_order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(asymmetric_low_order.value)

    @asymmetric_low_order.setter
    def asymmetric_low_order(self, asymmetric_low_order: int):
        status = self._dll.setAsymmetricLowOrder(asymmetric_low_order)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def asymmetric_high_order(self) -> int:
        """Order for high side of an asymmetric filter.
        The default is ''5''.

        Returns
        -------
        int
        """
        asymmetric_high_order = c_int()
        status = self._dll.getAsymmetricHighOrder(byref(asymmetric_high_order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(asymmetric_high_order.value)

    @asymmetric_high_order.setter
    def asymmetric_high_order(self, asymmetric_high_order: int):
        status = self._dll.setAsymmetricHighOrder(asymmetric_high_order)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def asymmetric_low_stop_band_ratio(self) -> str:
        """Stop-band ratio for low side of an asymmetric filter.
        The default is ''1.2''.

        Returns
        -------
        str
        """
        asymmetric_low_stop_band_ratio_string = self._dll_interface.get_string(self._dll.getAsymmetricLowStopbandRatio)
        return asymmetric_low_stop_band_ratio_string

    @asymmetric_low_stop_band_ratio.setter
    def asymmetric_low_stop_band_ratio(self, asymmetric_low_stop_band_ratio_string):
        self._dll_interface.set_string(
            self._dll.setAsymmetricLowStopbandRatio,
            asymmetric_low_stop_band_ratio_string,
        )

    @property
    def asymmetric_high_stop_band_ratio(self) -> str:
        """Stop-band ratio for high side of an asymmetric filter.
        The default is ''1.2''.

        Returns
        -------
        str
        """
        asymmetric_high_stop_band_ratio_string = self._dll_interface.get_string(
            self._dll.getAsymmetricHighStopbandRatio
        )
        return asymmetric_high_stop_band_ratio_string

    @asymmetric_high_stop_band_ratio.setter
    def asymmetric_high_stop_band_ratio(self, asymmetric_high_stop_band_ratio_string):
        self._dll_interface.set_string(
            self._dll.setAsymmetricHighStopbandRatio,
            asymmetric_high_stop_band_ratio_string,
        )

    @property
    def asymmetric_low_stop_band_attenuation_db(self) -> str:
        """Stop-band attenuation for low side of an asymmetric filter.
        The default is ''60 dB''.

        Returns
        -------
        str
        """
        asymmetric_low_stop_band_attenuation_db_string = self._dll_interface.get_string(
            self._dll.getAsymmetricLowStopbandAttenuationdB
        )
        return asymmetric_low_stop_band_attenuation_db_string

    @asymmetric_low_stop_band_attenuation_db.setter
    def asymmetric_low_stop_band_attenuation_db(self, asymmetric_low_stop_band_attenuation_db_string):
        self._dll_interface.set_string(
            self._dll.setAsymmetricLowStopbandAttenuationdB,
            asymmetric_low_stop_band_attenuation_db_string,
        )

    @property
    def asymmetric_high_stop_band_attenuation_db(self) -> str:
        """Stop-band attenuation for high side of an asymmetric filter.
        The default is ''60 dB''.

        Returns
        -------
        str
        """
        asymmetric_high_stop_band_attenuation_db_string = self._dll_interface.get_string(
            self._dll.getAsymmetricHighStopbandAttenuationdB
        )
        return asymmetric_high_stop_band_attenuation_db_string

    @asymmetric_high_stop_band_attenuation_db.setter
    def asymmetric_high_stop_band_attenuation_db(self, asymmetric_high_stop_band_attenuation_db_string):
        self._dll_interface.set_string(
            self._dll.setAsymmetricHighStopbandAttenuationdB,
            asymmetric_high_stop_band_attenuation_db_string,
        )

    @property
    def gaussian_transition(self) -> GaussianTransition:
        """Gaussian filter transition option.
        The default is ''TRANSITION_NONE''.

        Returns
        -------
        :enum:`GaussianTransition`
        """
        type_string = self._dll_interface.get_string(self._dll.getGaussianTransition)
        type_string = "TRANSITION_" + type_string
        return self._dll_interface.string_to_enum(GaussianTransition, type_string)

    @gaussian_transition.setter
    def gaussian_transition(self, gaussian_transition: GaussianTransition):
        string_value = self._dll_interface.enum_to_string(gaussian_transition)
        self._dll_interface.set_string(self._dll.setGaussianTransition, string_value)

    @property
    def gaussian_bessel_reflection(self) -> GaussianBesselReflection:
        """Gaussian or Bessel filter reflection option.
        The default is ''OPTION_1''.

        Returns
        -------
        :enum:`GaussianBesselReflection`
        """
        index = c_int()
        gaussian_bessel_reflection = list(GaussianBesselReflection)
        status = self._dll.getGaussianBesselReflection(byref(index))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        gaussian_bessel_reflection = gaussian_bessel_reflection[index.value]
        return gaussian_bessel_reflection

    @gaussian_bessel_reflection.setter
    def gaussian_bessel_reflection(self, column: GaussianBesselReflection):
        status = self._dll.setGaussianBesselReflection(column.value)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def even_order(self) -> bool:
        """Flag indicating if the even order mode for a filter with even orders is enabled.

        Returns
        -------
        bool
        """
        even_order = c_bool()
        status = self._dll.getEvenOrderMode(byref(even_order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(even_order.value)

    @even_order.setter
    def even_order(self, even_order: bool):
        status = self._dll.setEvenOrderMode(even_order)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def even_order_refl_zero(self) -> bool:
        """Flag indicating if the even order reflection zeros translation to 0 is enabled.

        Returns
        -------
        bool
        """
        even_order_refl_zero = c_bool()
        status = self._dll.getEvenReflZeroTo0(byref(even_order_refl_zero))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(even_order_refl_zero.value)

    @even_order_refl_zero.setter
    def even_order_refl_zero(self, even_order_refl_zero: bool):
        status = self._dll.setEvenReflZeroTo0(even_order_refl_zero)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def even_order_trn_zero(self) -> bool:
        """Flag indicating if the even order reflection zeros translation to infinite is enabled.

        Returns
        -------
        bool
        """
        even_order_trn_zero = c_bool()
        status = self._dll.getEvenTrnZeroToInf(byref(even_order_trn_zero))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(even_order_trn_zero.value)

    @even_order_trn_zero.setter
    def even_order_trn_zero(self, even_order_trn_zero: bool):
        status = self._dll.setEvenTrnZeroToInf(even_order_trn_zero)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def constrict_ripple(self) -> bool:
        """Flag indicating if the equiripple constriction is enabled.

        Returns
        -------
        bool
        """
        constrict_ripple = c_bool()
        status = self._dll.getConstrictRipple(byref(constrict_ripple))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(constrict_ripple.value)

    @constrict_ripple.setter
    def constrict_ripple(self, constrict_ripple: bool):
        status = self._dll.setConstrictRipple(constrict_ripple)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def single_point_ripple(self) -> bool:
        """Flag indicating if the ripple confinement to a single frequency point is enabled.

        Returns
        -------
        bool
        """
        single_point_ripple = c_bool()
        status = self._dll.getSinglePointRipple(byref(single_point_ripple))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(single_point_ripple.value)

    @single_point_ripple.setter
    def single_point_ripple(self, single_point_ripple: bool):
        self._dll.setSinglePointRipple(single_point_ripple)

    @property
    def half_band_ripple(self) -> bool:
        """Flag indicating if the ripple with half of the zeros in the given band is enabled.

        Returns
        -------
        bool
        """
        half_band_point_ripple = c_bool()
        status = self._dll.getHalfBandRipple(byref(half_band_point_ripple))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(half_band_point_ripple.value)

    @half_band_ripple.setter
    def half_band_ripple(self, half_band_ripple: bool):
        status = self._dll.setHalfBandRipple(half_band_ripple)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def constrict_ripple_percent(self) -> str:
        """Filter ripple constriction percentage.
        The default is ''50%''.

        Returns
        -------
        str
        """
        constrict_ripple_percent_string = self._dll_interface.get_string(self._dll.getRippleConstrictionPercent)
        return constrict_ripple_percent_string

    @constrict_ripple_percent.setter
    def constrict_ripple_percent(self, constrict_ripple_percent_string):
        self._dll_interface.set_string(self._dll.setRippleConstrictionPercent, constrict_ripple_percent_string)

    @property
    def ripple_constriction_band(self) -> RippleConstrictionBandSelect:
        """Filter ripple constriction band option.
        The default is ''STOP''.

        Returns
        -------
        :enum:`RippleConstrictionBandSelect`
        """
        type_string = self._dll_interface.get_string(self._dll.getRippleConstrictionBandSelect)
        return self._dll_interface.string_to_enum(RippleConstrictionBandSelect, type_string)

    @ripple_constriction_band.setter
    def ripple_constriction_band(self, ripple_constriction_band: RippleConstrictionBandSelect):
        string_value = self._dll_interface.enum_to_string(ripple_constriction_band)
        self._dll_interface.set_string(self._dll.setRippleConstrictionBandSelect, string_value)

    @property
    def single_point_ripple_inf_zeros(self) -> SinglePointRippleInfZeros:
        """Filter number of single point ripple infinite zeros.
        The default is ''RIPPLE_INF_ZEROS_1''.

        Returns
        -------
        :enum:`SinglePointRippleInfZeros`
        """
        type_string = self._dll_interface.get_string(self._dll.getSinglePointRippleNoninfiniteZeros)
        type_string = "RIPPLE_INF_ZEROS_" + type_string
        return self._dll_interface.string_to_enum(SinglePointRippleInfZeros, type_string)

    @single_point_ripple_inf_zeros.setter
    def single_point_ripple_inf_zeros(self, single_point_ripple_inf_zeros: SinglePointRippleInfZeros):
        string_value = self._dll_interface.enum_to_string(single_point_ripple_inf_zeros)
        self._dll_interface.set_string(self._dll.setSinglePointRippleNoninfiniteZeros, string_value)

    @property
    def delay_equalizer(self) -> bool:
        """Flag indicating if the delay equalizer is enabled.

        Returns
        -------
        bool
        """
        delay_equalizer = c_bool()
        status = self._dll.getDelayEqualizer(byref(delay_equalizer))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(delay_equalizer.value)

    @delay_equalizer.setter
    def delay_equalizer(self, delay_equalizer: bool):
        self._dll.setDelayEqualizer(delay_equalizer)

    @property
    def delay_equalizer_order(self) -> int:
        """Filter delay equalizer order.
        The default is ''2''.

        Returns
        -------
        int
        """
        delay_equalizer_order = c_int()
        status = self._dll.getDelayEqualizerOrder(byref(delay_equalizer_order))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(delay_equalizer_order.value)

    @delay_equalizer_order.setter
    def delay_equalizer_order(self, delay_equalizer_order: int):
        status = self._dll.setDelayEqualizerOrder(delay_equalizer_order)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def standard_delay_equ_pass_band_attenuation(self) -> bool:
        """Flag indicating if the standard delay equalizer attenuation is enabled.

        Returns
        -------
        bool
        """
        standard_dealy_equ_cut = c_bool()
        status = self._dll.getStandardDelayEquCut(byref(standard_dealy_equ_cut))
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(standard_dealy_equ_cut.value)

    @standard_delay_equ_pass_band_attenuation.setter
    def standard_delay_equ_pass_band_attenuation(self, standard_delay_equ_pass_band_attenuation: bool):
        status = self._dll.setStandardDelayEquCut(standard_delay_equ_pass_band_attenuation)
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def standard_delay_equ_pass_band_attenuation_value_db(self) -> str:
        """Filter standard delay equalizer cut Off attenuation in dB.
        The default is ''3.01 dB''.

        Returns
        -------
        str
        """
        delay_equcutoff_attenuation_db_string = self._dll_interface.get_string(self._dll.getDelayEquCutoffAttenuationdB)
        return delay_equcutoff_attenuation_db_string

    @standard_delay_equ_pass_band_attenuation_value_db.setter
    def standard_delay_equ_pass_band_attenuation_value_db(
        self, standard_delay_equ_pass_band_attenuation_value_db_string
    ):
        self._dll_interface.set_string(
            self._dll.setDelayEquCutoffAttenuationdB,
            standard_delay_equ_pass_band_attenuation_value_db_string,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ctypes import POINTER
from ctypes import byref
from ctypes import c_bool
from ctypes import c_char_p
from ctypes import c_int
from ctypes import create_string_buffer
from enum import Enum

import pyaedt


class ComplexTerminationDefinition(Enum):
    """Selects type of complex presentation.

    **Attributes:**

    - POLAR: Represents polar definition.
    - CARTESIAN: Represents Cartesian definition.
    - PARALLEL: Represents parallel definition with real entry parallel to imaginary entry.
    - REAL: Represents only real impedance definition.
    """

    POLAR = 0
    CARTESIAN = 1
    PARALLEL = 2
    REAL = 3


class ComplexReactanceType(Enum):
    """Selects type of complex impedance as reactance, equivalent inductance, or equivalent capacitance.

    **Attributes:**

    - REAC: Represents pure reactance of complex impedance.
    - IND: Represents equivalent inductance in henries.
    - CAP: Represents equivalent capacitance in farads.
    """

    REAC = 0
    IND = 1
    CAP = 2


class TerminationType(Enum):
    """Selects either source or load complex impedance table.

    **Attributes:**

    - SOURCE: Represents source impedenace table.
    - LOAD: Represents load impedenace table.
    """

    SOURCE = 0
    LOAD = 1


class LumpedTerminationImpedance:
    """Manipulates access to the entries of source and load complex impedance table.

    This class allows you to enter, edit, or remove the entries of source and load complex impedance table.
    """

    def __init__(self, table_type):
        self._dll = pyaedt.filtersolutions_core._dll_interface()._dll
        self._dll_interface = pyaedt.filtersolutions_core._dll_interface()
        self._define_termination_impedance_dll_functions()
        self.table_type = table_type

    def _define_termination_impedance_dll_functions(self):
        """Define C++ API DLL functions."""
        self._dll.getComplexTableRowCount.argtypes = [POINTER(c_int), c_bool]
        self._dll.getComplexTableRowCount.restype = c_int

        self._dll.getComplexTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_char_p,
            c_bool,
            c_int,
        ]
        self._dll.getComplexTableRow.restype = c_int

        self._dll.updateComplexTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.updateComplexTableRow.restype = c_int

        self._dll.appendComplexTableRow.argtypes = [
            c_char_p,
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.appendComplexTableRow.restype = c_int

        self._dll.insertComplexTableRow.argtypes = [
            c_int,
            c_char_p,
            c_char_p,
            c_char_p,
            c_bool,
        ]
        self._dll.insertComplexTableRow.restype = c_int

        self._dll.removeComplexTableRow.argtypes = [c_int, c_bool]
        self._dll.removeComplexTableRow.restype = c_int

        self._dll.setLumpedComplexDefinition.argtypes = [c_char_p, c_bool]
        self._dll.setLumpedComplexDefinition.restype = c_int
        self._dll.getLumpedComplexDefinition.argtypes = [c_char_p, c_bool, c_int]
        self._dll.getLumpedComplexDefinition.restype = c_int

        self._dll.setLumpedComplexReactanceType.argtypes = [c_char_p, c_bool]
        self._dll.setLumpedComplexReactanceType.restype = c_int
        self._dll.getLumpedComplexReactanceType.argtypes = [c_char_p, c_bool, c_int]
        self._dll.getLumpedComplexReactanceType.restype = c_int

        self._dll.setLumpedComplexImpCompensateEnabled.argtypes = [c_bool, c_bool]
        self._dll.setLumpedComplexImpCompensateEnabled.restype = c_int
        self._dll.getLumpedComplexImpCompensateEnabled.argtypes = [
            POINTER(c_bool),
            c_bool,
        ]
        self._dll.getLumpedComplexImpCompensateEnabled.restype = c_int

        self._dll.setLumpedComplexCompOrder.argtypes = [c_int, c_bool]
        self._dll.setLumpedComplexCompOrder.restype = c_int
        self._dll.getLumpedComplexCompOrder.argtypes = [POINTER(c_int), c_bool]
        self._dll.getLumpedComplexCompOrder.restype = c_int

    def table_type_to_bool(self):
        """Set a flag to recognize source and load complex table.

        Returns
        -------
        bool
        """
        if self.table_type.value == TerminationType.SOURCE.value:
            return False
        else:
            return True

    @property
    def row_count(self) -> int:
        """Count of the accumulated complex impedances in the complex impedances's table.
        The default is ``3``.

        Returns
        -------
        int
        """
        table_row_count = c_int()
        status = self._dll.getComplexTableRowCount(byref(table_row_count), self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(table_row_count.value)

    def row(self, row_index):
        """Get frequency and complex impedance values from a row in the complex impedance table.

        Parameters
        ----------
        row_index: int
            Row index on complex impedance table, starting at ``0`` and with a maximum value of ``99``.

        Returns
        -------
        tuple
            The tuple contains three strings. The first is the frequency value,
            the second is the real part of the complex impedance,
            and the third is the imaginary part of the complex impedance.
        """
        frequency_value_buffer = create_string_buffer(100)
        real_value_buffer = create_string_buffer(100)
        imag_value_buffer = create_string_buffer(100)
        status = self._dll.getComplexTableRow(
            row_index,
            frequency_value_buffer,
            real_value_buffer,
            imag_value_buffer,
            self.table_type_to_bool(),
            100,
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        frequency_value_string = frequency_value_buffer.value.decode("utf-8")
        real_value_string = real_value_buffer.value.decode("utf-8")
        imag_value_string = imag_value_buffer.value.decode("utf-8")
        return frequency_value_string, real_value_string, imag_value_string

    def update_row(self, row_index, frequency="", real="", imag=""):
        """Update frequency and complex impedance at a specified index in the complex impedance table.

        Parameters
        ----------
        row_index: int
            Row index on complex impedance table, starting at ``0`` and with a maximum value of ``99``.
        frequency: str, optional
            The frequency value to update. If not specified, it remains unchanged.
        real: str, optional
            The real part of the complex impedance to update. If not specified, it remains unchanged.
        imag: str, optional
            The imaginary part of the complex impedance to update. If not specified, it remains unchanged.
        """
        frequency_bytes_value = bytes(frequency, "ascii")
        real_bytes_value = bytes(real, "ascii")
        imag_bytes_value = bytes(imag, "ascii")
        status = self._dll.updateComplexTableRow(
            row_index,
            frequency_bytes_value,
            real_bytes_value,
            imag_bytes_value,
            self.table_type_to_bool(),
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def append_row(self, frequency, real, imag):
        """Append frequency and complex impedance values to the last row of
        both the source and load complex impedance table.


        Parameters
        ----------
        frequency: str
            The frequency value to append.
        real: str
            The real part of the complex impedance to append.
        imag: str
            The imaginary part of the complex impedance to append.
        """
        frequency_bytes_value = bytes(frequency, "ascii")
        real_bytes_value = bytes(real, "ascii")
        imag_bytes_value = bytes(imag, "ascii")
        status = self._dll.appendComplexTableRow(
            frequency_bytes_value,
            real_bytes_value,
            imag_bytes_value,
            self.table_type_to_bool(),
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def insert_row(self, row_index, frequency, real, imag):
        """Insert frequency and complex impedance values at a specified index in the complex impedance table.

        Parameters
        ----------
        row_index : int
            Row index in the complex impedance table, starting at ``0`` and with a maximum value of ``99``.
        frequency : str
            The frequency value to insert.
        real : str
            The real part of the complex impedance to insert.
        imag : str
            The imaginary part of the complex impedance to insert.
        """
        frequency_bytes_value = bytes(frequency, "ascii")
        real_bytes_value = bytes(real, "ascii")
        imag_bytes_value = bytes(imag, "ascii")
        status = self._dll.insertComplexTableRow(
            row_index,
            frequency_bytes_value,
            real_bytes_value,
            imag_bytes_value,
            self.table_type_to_bool(),
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    def remove_row(self, row_index):
        """Remove frequency and complex impedance at a specified index from the complex impedance table.

        Parameters
        ----------
        row_index : int
            Row index in the complex impedance table, starting at ``0`` and with a maximum value of ``99``.
        """
        status = self._dll.removeComplexTableRow(row_index, self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def complex_definition(self) -> ComplexTerminationDefinition:
        """Definition type of complex impedance in the complex impedance table.
        The default is ``Cartesian``.

        Returns
        -------
        :enum:`ComplexTerminationDefinition`
        """
        type_string_buffer = create_string_buffer(100)
        status = self._dll.getLumpedComplexDefinition(
            type_string_buffer,
            self.table_type_to_bool(),
            100,
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        type_string = type_string_buffer.value.decode("utf-8")
        return self._dll_interface.string_to_enum(ComplexTerminationDefinition, type_string)

    @complex_definition.setter
    def complex_definition(self, complex_definition: ComplexTerminationDefinition):
        string_value = self._dll_interface.enum_to_string(complex_definition)
        string_bytes_value = bytes(string_value, "ascii")
        status = self._dll.setLumpedComplexDefinition(string_bytes_value, self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def reactance_type(self) -> ComplexReactanceType:
        """Reactance type of complex impedance in the complex impedance table.
        The default is ``reactance``.

        Returns
        -------
        :enum:`ComplexReactanceType`
        """

        type_string_buffer = create_string_buffer(100)
        status = self._dll.getLumpedComplexReactanceType(
            type_string_buffer,
            self.table_type_to_bool(),
            100,
        )
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        type_string = type_string_buffer.value.decode("utf-8")
        return self._dll_interface.string_to_enum(ComplexReactanceType, type_string)

    @reactance_type.setter
    def reactance_type(self, reactance_type: ComplexReactanceType):
        string_value = self._dll_interface.enum_to_string(reactance_type)
        string_bytes_value = bytes(string_value, "ascii")
        status = self._dll.setLumpedComplexReactanceType(string_bytes_value, self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def compensation_enabled(self) -> bool:
        """Flag indicating if the impedance compensation is enabled.

        Returns
        -------
        bool
        """
        compensation_enabled = c_bool()
        status = self._dll.getLumpedComplexImpCompensateEnabled(byref(compensation_enabled), self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return bool(compensation_enabled.value)

    @compensation_enabled.setter
    def compensation_enabled(self, compensation_enabled: bool):
        status = self._dll.setLumpedComplexImpCompensateEnabled(compensation_enabled, self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)

    @property
    def compensation_order(self) -> int:
        """Order of impedance compensation.
        The default is` ``2``.

        Returns
        -------
        int
        """
        compensation_order = c_int()
        status = self._dll.getLumpedComplexCompOrder(byref(compensation_order), self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
        return int(compensation_order.value)

    @compensation_order.setter
    def compensation_order(self, compensation_order: int):
        status = self._dll.setLumpedComplexCompOrder(compensation_order, self.table_type_to_bool())
        pyaedt.filtersolutions_core._dll_interface().raise_error(status)
